// Copyright (C) 2004 by OpenFlower Team
// http://sourceforge.net/projects/openflower/
// Licensed under the GNU GPL Version 2.

#include <iostream>
//#include <time.h>

#include "AlgoScalarDiff.h"
#include "MyVector.h"
#include "MyMacros.h"
#include "Simulation.h"
#include "Variable.h"
#include "Medium.h"

#include "EqnTemperatureDiffusion.h"
#include "OpDiffusion.h"

using namespace std;



AlgoScalarDiff::AlgoScalarDiff()
  {
  nbEqn=0;
  pEqn.resize(0);
  }



AlgoScalarDiff::~AlgoScalarDiff()
  {
  }



/*!\brief Executes the Algorithm.
 *
 */
void AlgoScalarDiff::execute()
  {

//  clock_t t1=clock();
  EqnTemperatureDiffusion *eq0 = dynamic_cast<EqnTemperatureDiffusion*>(pEqn[0]);
//  clock_t t2=clock();
  
  if(!eq0)
    {
    ERROR_IN_FILE;
    cerr << "Specified Equation is not consistant with Algorithm"<<endl;
    cerr << "Expecting 1 Equation: Scalar Diffusion Equation."<<endl;
    cerr << "Exiting"<<endl;
    exit(-1);
    }
  
//  clock_t t3=clock();
  OpDiffusion& op0 = dynamic_cast<OpDiffusion&>(eq0->getOperator(0));
  double dt = pSimulation->getTimeStep();
//  clock_t t4=clock();

  Variable& rT       = eq0->getPresent();
  Variable& rTFuture = eq0->getFuture();
  const Variable& rLambda  = eq0->getMedium().getLambda();
  const Variable& rRho  = eq0->getMedium().getRho();
  const Variable& rCp  = eq0->getMedium().getCp();
//  clock_t t6=clock();

  Variable alpha(rLambda);
  alpha/=rRho;
  alpha/=rCp;
//  clock_t t8=clock();


  Variable diff;

//  clock_t t9=clock();
  op0.evaluate(alpha, rT, diff);
//  clock_t t10=clock();

  Variable RHS0(diff);
//  clock_t t11=clock();

// Explicit Euler Time advancement
  RHS0*=dt;
  RHS0+=rT;
//  clock_t t115=clock();

  rTFuture = RHS0;
//  clock_t t12=clock();
/*  cerr << endl<<"####### Total CPUt for Algo. iteration : "<<long(t12-t1)/(double)CLOCKS_PER_SEC<<" s. ########"<<endl;
  cerr << " - Recup pEqn: "<<long(t2-t1)/(double)CLOCKS_PER_SEC<<" s. ["<< (t2-t1)*100./(t12-t1) << "%]"<<endl;
  cerr << " - Recup op et dt: "<<(t4-t3)/(double)CLOCKS_PER_SEC<<" s. ["<< (t4-t3)*100./(t12-t1) << "%]"<<endl;
  cerr << " - Recup all vars: "<<(t6-t4)/(double)CLOCKS_PER_SEC<<" s. ["<< (t6-t4)*100./(t12-t1) << "%]"<<endl;
  cerr << " - Calc alpha: "<<(t8-t6)/(double)CLOCKS_PER_SEC<<" s. ["<< (t8-t6)*100./(t12-t1) << "%]"<<endl;
  cerr << " - Create var: "<<(t9-t8)/(double)CLOCKS_PER_SEC<<" s. ["<< (t9-t8)*100./(t12-t1) << "%]"<<endl;
  cerr << " - Calc diff: "<<(t10-t9)/(double)CLOCKS_PER_SEC<<" s. ["<< (t10-t9)*100./(t12-t1) << "%]"<<endl;
  cerr << " - Create RHS with diff: "<<(t11-t10)/(double)CLOCKS_PER_SEC<<" s. ["<< (t11-t10)*100./(t12-t1) << "%]"<<endl;
  cerr << " - Calc RHS (2 ops): "<<(t115-t11)/(double)CLOCKS_PER_SEC<<" s. ["<< (t115-t11)*100./(t12-t1) << "%]"<<endl;
  cerr << " - fut=RHS: "<<(t12-t115)/(double)CLOCKS_PER_SEC<<" s. ["<< (t12-t115)*100./(t12-t1) << "%]"<<endl<<endl<<endl;
*/
  }



double AlgoScalarDiff::getStabilityTimeStep()
  {
  EqnTemperatureDiffusion & eq0 = dynamic_cast<EqnTemperatureDiffusion&>(*pEqn[0]);
  
  const Variable& rLambda  = eq0.getMedium().getLambda();
  const Variable& rRho     = eq0.getMedium().getRho();
  const Variable& rCp      = eq0.getMedium().getCp();
  Variable alpha(rLambda);
  alpha/=rRho;
  alpha/=rCp;

  OpDiffusion& op0 = dynamic_cast<OpDiffusion&>(eq0.getOperator(0));
  
  return op0.getStabilityTimeStep(alpha);
  }

