// Copyright (C) 2004 by OpenFlower Team
// http://sourceforge.net/projects/openflower/
// Licensed under the GNU GPL Version 2.

#ifndef ALGORITHM_H
#define ALGORITHM_H

#include <stdlib.h>
#include <stdio.h>
#include <string>

#include "MyVector.h"

using namespace std;

class Equation;
class Simulation;

class Algorithm {
public:
  Algorithm();
  virtual ~Algorithm();

  void read(ifstream& inputFile); //!< Reads information from the input FLW file.

  virtual void execute(){}        //!< Executes the Algorithm.
  virtual double getStabilityTimeStep(){} //! Calculates the Stability Time Step specific to this Algorithm.
  int getNbEqn(){return nbEqn;}
  inline string getLabelOf(int i) {return labels[i];}

  void associateEquation(int i, Equation* eqn);
  void associateSimulation(Simulation* _pSimulation){ pSimulation = _pSimulation; }
  
protected:
  MyVector<Equation*> pEqn; //!< The list of all Equations involved in the Algorithm.
  int nbEqn;                //!< The number (>=1) of Equations involved in the Algorithm.
  MyVector<string> labels;
  Simulation* pSimulation;  // A Pointer to the Simulation.
};

#endif
