// Copyright (C) 2004 by OpenFlower Team
// http://sourceforge.net/projects/openflower/
// Licensed under the GNU GPL Version 2.

#ifndef BOUNDARYCONDITION_H
#define BOUNDARYCONDITION_H

#include <stdlib.h>
#include <string>

#include "MyMatrix.h"
#include "Parser.h"
#include "Frontier.h"

using namespace std;

class Mesh;
class Equation;
class BoundaryCondition
{
public: 
  BoundaryCondition();
  ~BoundaryCondition();
  
  inline int getNbFaces() {return pFrontier->getNbFaces();};
  inline int getNbFirstFace() {return pFrontier->getNbFirstFace();};
  void associateMesh(const Mesh* _pMesh) {pMesh = _pMesh;};
  void associateEqn(const Equation* _pEquation) {pEquation = _pEquation;};
  void associateFrontier(const string _Name);
  virtual void read(ifstream& data_file)=0;
  virtual double getValue(const int faceNb, const int comp, const MyVector<double>& gradElement)=0;
  virtual double getNormalGrad(const int faceNb, const int comp, const MyVector<double>& gradElement)=0;

  
protected:
  MyMatrix<double> values;    //!< Values (scalar or vector) affected to the Boundary Condition on each face.
  const Mesh* pMesh;
  const Equation* pEquation;  //!< The Equation to which the Boundary Condition is affected.
  const Frontier* pFrontier;  //!< The Frontier to which the Boundary Condition is affected.
  
};

#endif
