// Copyright (C) 2004 by OpenFlower
// http://sourceforge.net/projects/openflower/
// Licensed under the GNU GPL Version 2.

#include <iostream>

#include "EqnAllenCahn.h"

#include "OpDiffusion.h"

#include "BCTemperatureImposed.h"
#include "BCHeatFlux.h"
#include "Mesh.h"
#include "MyMacros.h"

// For testing purpose only
#include "PostProcFieldGmsh.h"
#include <time.h>
// For testing purpose only

using namespace std;

////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
EqnAllenCahn::EqnAllenCahn()
  {
  name="EQ_ALLEN_CAHN";
  present = new Variable;
  future = new Variable;
  present->setLocation("cell");
  present->setName("Phi");
  future->setLocation("cell");
  future->setName("Phi");
  nbBC=0;
  nbOp=0;
  lambda.setLocation("cell");
  lambda.setName("Lambda");
  k.setLocation("cell");
  k.setName("K");
  gamma.setLocation("cell");
  gamma.setName("Gamma");
  beta.setLocation("cell");
  beta.setName("Beta");
  }



////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
void EqnAllenCahn::readVirtual(ifstream& inputFile, string & keyword)
  {
  MyVector<string> knownKeywords;
  knownKeywords.resize(6);
  knownKeywords[0]="boundary_condition";
  knownKeywords[1]="operator";
  knownKeywords[2]="lambda";
  knownKeywords[3]="gamma";
  knownKeywords[4]="beta";
  knownKeywords[5]="k";

  switch(knownKeywords.search(keyword))
    {
    case  0 :
            {
	    addBC(inputFile);
	    boundaryConditions[nbBC-1]->read(inputFile);
            break;
            }
    case  1 :
            {
	    addOperator(inputFile);
            operators[nbOp-1]->associateMesh(pMesh);
            operators[nbOp-1]->initialize();
            break;
            }
    case  2 :
            {
            lambda.resize(pMesh->getNbElements());
	    initialize(lambda, inputFile);
            break;
            }
    case  3 :
            {
            gamma.resize(pMesh->getNbElements());
	    initialize(gamma, inputFile);
            break;
            }
    case  4 :
            {
            beta.resize(pMesh->getNbElements());
	    initialize(beta, inputFile);
            break;
            }
    case  5 :
            {
            k.resize(pMesh->getNbElements());
	    initialize(k, inputFile);
            break;
            }
    case -1 :
            {
            ERROR_IN_FILE;
            cerr << keyword << " is an unknown keyword!" << endl;
            cerr << "  Known keywords are: ";
            for (unsigned int i=1;i<knownKeywords.size();i++)
              {
              cerr << knownKeywords[i] << " ";
              }
            cerr << endl;
            exit(-1);
            break;
            }
    default: break;  // Impossible case
    }
  };



////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
void EqnAllenCahn::addBC(ifstream& inputFile)
  {
  string keyword;
  MyVector<string> knownBCs;
  knownBCs.resize(2);
  knownBCs[0]="temperature";
  knownBCs[1]="heat_flux";

  inputFile >> keyword;

  switch(knownBCs.search(keyword))
    {
    case 0:
          {
          nbBC++;
          boundaryConditions.resize(nbBC);
          boundaryConditions[nbBC-1] = new BCTemperatureImposed;
          break;
          }
    case 1:
          {
          nbBC++;
          boundaryConditions.resize(nbBC);
          boundaryConditions[nbBC-1] = new BCHeatFlux;
          break;
          }
    case -1:
           {
           ERROR_IN_FILE;
           cerr << "Specified boundary condition: "<<keyword<< " is not available at the moment..." << endl;
           cerr << "  Possible boundary conditions are: ";
           for (unsigned int i=0;i<knownBCs.size();i++)
             {
	     cout << knownBCs[i] << " ";
	     }
           cout << endl;
           exit(-1);
           break;
           }
    default: break;
    }
  boundaryConditions[nbBC-1]->associateMesh(pMesh);
  boundaryConditions[nbBC-1]->associateEqn(this);
  }



////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
void EqnAllenCahn::addOperator(ifstream& inputFile)
  {
  string keyword;
  MyVector<string> possibleOperators;
  possibleOperators.resize(1);
  possibleOperators[0]="diffusion";

  inputFile >> keyword;

  switch(possibleOperators.search(keyword))
    {
    case 0:
          {
          nbOp++;
          operators.resize(nbOp);
          operators[nbOp-1] = new OpDiffusion;
          break;
          }
    case -1:
           {
           ERROR_IN_FILE;
           cerr << "Specified operator: "<<keyword<< " is impossible for EqnAllenCahn..." << endl;
           cerr << "  Possible operators are: ";
           for (unsigned int i=0;i<possibleOperators.size();i++)
             {
             cerr << possibleOperators[i] << " ";
             }
           cerr << endl;
           exit(-1);
           break;
           }
    default: break;
    }
  operators[nbOp-1]->associateMesh(pMesh);
  operators[nbOp-1]->associateEqn(this);
  }

  
  
void EqnAllenCahn::addsItsVariables4PostProcessing(map<string,const Variable*> & variablesMap)
  {
  string labelOfVariable;
  labelOfVariable=label;
  labelOfVariable+=".Phi";
  variablesMap[labelOfVariable]=present;
  
  labelOfVariable=label;
  labelOfVariable+=".Lambda";
  variablesMap[labelOfVariable]= &lambda;
  
  labelOfVariable=label;
  labelOfVariable+=".Gamma";
  variablesMap[labelOfVariable]=&gamma;
  
  labelOfVariable=label;
  labelOfVariable+=".Beta";
  variablesMap[labelOfVariable]=&beta;
  
  labelOfVariable=label;
  labelOfVariable+=".K";
  variablesMap[labelOfVariable]=&k;
  }
