// Copyright (C) 2004 by OpenFlower
// http://sourceforge.net/projects/openflower/
// Licensed under the GNU GPL Version 2.

#include <iostream>

#include "EqnTemperatureDiffusion.h"
#include "OpDiffusion.h"

#include "BCTemperatureImposed.h"
#include "BCHeatFlux.h"
#include "Mesh.h"
#include "MyMacros.h"


using namespace std;

////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
EqnTemperatureDiffusion::EqnTemperatureDiffusion()
  {
  name="EQ_TEMPERATURE_DIFFUSION";
  present = new Variable;
  future = new Variable;
  present->setLocation("cell");
  present->setName("Temperature");
  future->setLocation("cell");
  future->setName("Temperature");
  nbBC=0;
  nbOp=0;
  }



////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
void EqnTemperatureDiffusion::readVirtual(ifstream& inputFile, string & keyword)
  {
  MyVector<string> knownKeywords;
  knownKeywords.resize(2);
  knownKeywords[0]="boundary_condition";
  knownKeywords[1]="operator";
//  knownKeywords[2]="initial_condition";

  present->resize(pMesh->getNbElements());
  future->resize(pMesh->getNbElements());

  switch(knownKeywords.search(keyword))
    {
    case  0 :
            {
	    addBC(inputFile);
	    boundaryConditions[nbBC-1]->read(inputFile);
            break;
            }
    case  1 :
            {
	    addOperator(inputFile);
            operators[nbOp-1]->associateMesh(pMesh);
            operators[nbOp-1]->initialize();
            break;
            }
    case -1 :
            {
            ERROR_IN_FILE;
            cerr << keyword << " is an unknown keyword!" << endl;
            cerr << "  Known keywords are:";
            for (unsigned int i=1;i<knownKeywords.size();i++)
              {
              cerr << " " << knownKeywords[i];
              }
            cerr << endl;
            exit(-1);
            break;
            }
    default: break;  // Impossible case
    }
  };



////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
void EqnTemperatureDiffusion::addBC(ifstream& inputFile)
  {
  string keyword;
  MyVector<string> knownBCs;
  knownBCs.resize(2);
  knownBCs[0]="temperature";
  knownBCs[1]="heat_flux";

  inputFile >> keyword;

  switch(knownBCs.search(keyword))
    {
    case 0:
          {
          nbBC++;
          boundaryConditions.resize(nbBC);
          boundaryConditions[nbBC-1] = new BCTemperatureImposed;
          break;
          }
    case 1:
          {
          nbBC++;
          boundaryConditions.resize(nbBC);
          boundaryConditions[nbBC-1] = new BCHeatFlux;
          break;
          }
    case -1:
           {
           ERROR_IN_FILE;
           cerr << "Specified boundary condition: "<<keyword<< " is not available at the moment..." << endl;
           cerr << "  Possible boundary conditions are:";
           for (unsigned int i=0;i<knownBCs.size();i++)
             {
	     cerr << " " << knownBCs[i];
	     }
           cerr << endl;
           exit(-1);
           break;
           }
    default: break;
    }
  boundaryConditions[nbBC-1]->associateMesh(pMesh);
  boundaryConditions[nbBC-1]->associateEqn(this);
  }



////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
void EqnTemperatureDiffusion::addOperator(ifstream& inputFile)
  {
  string keyword;
  MyVector<string> possibleOperators;
  possibleOperators.resize(1);
  possibleOperators[0]="diffusion";

  inputFile >> keyword;

  switch(possibleOperators.search(keyword))
    {
    case 0:
          {
          nbOp++;
          operators.resize(nbOp);
          operators[nbOp-1] = new OpDiffusion;
          break;
          }
    case -1:
           {
           ERROR_IN_FILE;
           cerr << "Specified operator: "<<keyword<< " is impossible for EqnTemperatureDiffusion..." << endl;
           cerr << "  Possible operators are: ";
           for (unsigned int i=0;i<possibleOperators.size();i++)
             {
	     cout << possibleOperators[i] << " ";
	     }
           cout << endl;
           exit(-1);
           break;
           }
    default: break;
    }
  operators[nbOp-1]->associateMesh(pMesh);
  operators[nbOp-1]->associateEqn(this);
  }

  
  
void EqnTemperatureDiffusion::addsItsVariables4PostProcessing(map<string,const Variable*> & variablesMap)
  {
  string labelOfVariable;
  labelOfVariable=label;
  labelOfVariable+=".Temperature";
  variablesMap[labelOfVariable]=present;
  }
