// Copyright (C) 2004 by OpenFlower Team
// http://sourceforge.net/projects/openflower/
// Licensed under the GNU GPL Version 2.

#include <iostream>
#include <time.h>

#include "MyMacros.h"
#include "Equation.h"
#include "Parser.h"
#include "Element.h"
#include "Mesh.h"
#include "Variable.h"
#include "Simulation.h"

using namespace std;

////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
Equation::Equation()
  {
  present = new Variable;
  future = new Variable;
  name="EQ";
  nbBC=0;
  nbOp=0;
  labelOfMedium  = "empty";
  };



////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
void Equation::read(ifstream& inputFile)
  {
  string keyword;

  inputFile >> label;
  cout << "Present equation has following user-label: "<<label<<endl;
  
  inputFile >> keyword;
  if(keyword != "\{")
    {
    ERROR_IN_FILE;
    cerr << "Expecting an opening \"\{\" before equation block!" << endl;
    cerr << "Also check if the equation label was defined just before opening \{" << endl;
    assert(0);
    exit(-1);
    }
  inputFile >> keyword;

  MyVector<string> knownKeywords;
  knownKeywords.resize(3);
  knownKeywords[0]="//";
  knownKeywords[1]="Initial_Condition";
  knownKeywords[2]="Medium";

  while (keyword!="}")
    {
    switch(knownKeywords.search(keyword))
      {
      case  0 : inputFile.ignore(255,'\n'); break;
      case  1 :
              {
// FIXXME (Vortexflow, 11 oct 2004) -> here we should take care when giving the dimensions of the variables...
// now, it only works for scalar variables... not for vectors like velocity!
// Should be corrected later on.
              present->resize(pMesh->getNbElements());
              future->resize(pMesh->getNbElements());
// END FIXXME
              initialize(*present,inputFile);
              break;
              }
      case  2 :
              {
              inputFile >> labelOfMedium;
              break;
              }
      case -1 : this->readVirtual(inputFile,keyword); break;
      default : break ;
      }
      inputFile >> keyword;
    }
  }



void Equation::advance()
 {
 for(int i = 0;i<present->getNbValues();i++)
   {
   present->setValue(future->getValue(i),i);
   }
 }



void Equation::initialize(Variable& variable,ifstream& inputFile)
  {
  string keyword;
  const MyVector<Element>& elements = pMesh->getRefElements();
  MyVector<double> XYZElement;
  XYZElement.resize(3);
  double result;

  inputFile >> keyword;
  if(keyword=="Expr")
    {
    for(int n=0;n<variable.getNbDimensions0();n++)
      {
      Parser expression;
      inputFile >> keyword;
      expression.makeTree(keyword);
  
      for(int i=0;i<variable.getNbValues();i++)
        {
        XYZElement = elements[i].getXYZ();
  
        expression.setVariable("X",XYZElement[0]);
        expression.setVariable("Y",XYZElement[1]);
        expression.setVariable("Z",XYZElement[2]);
        expression.evaluate(result);
        variable.setValue(result,i);
        }
      }
    }
  else if(keyword=="File")
    {
    inputFile >> keyword;
    ifstream savedFile(keyword.c_str());
    if (!savedFile)
      {
      ERROR_IN_FILE;
      cerr << "Error trying to recover saved data in File: '" << keyword << endl;
      cerr << "File not found!" << endl;
      exit(-1);
      }
// FIXXME (Vortexflow, 11 oct 2004) -> We should add some tests to see if everything went ok while recovering the data.
    getline(savedFile,keyword);
    getline(savedFile,keyword);
    getline(savedFile,keyword);
    getline(savedFile,keyword);
    getline(savedFile,keyword);
    for(int i=0;i<present->getNbValues();i++)
      {
      for(int j=0;j<present->getNbDimensions0();j++)
        {
        for(int k=0;k<present->getNbDimensions1();k++)
          {
          savedFile >> result;
          present->setValue(result,i,j,k);
          }
        }
      }
// END FIXXME
    }
  else
    {
    ERROR_IN_FILE;
    cerr << "Unexpected keyword for Variable Initialization: "<< keyword<<endl;
    cerr << "Expecting \'Expr\' followed by mathematical expression"<<endl;
    cerr << "Or        \'File\' followed by a file name."<<endl;
    exit(-1);
    }
  }



void Equation::finish()
  {
  string fileName=label;
  fileName+=".save";
  ofstream outputFile(fileName.c_str());
  time_t t;
  time(&t);
  outputFile << "// Generated by OpenFlower CFD Software on " << ctime(&t);
  outputFile << "// Field data of:" << endl;
  outputFile << "// - Equation : " << label << endl;
  outputFile << "// - Variable : " << present->getName() << endl;
  outputFile << "// - Time     : " << pSimulation->getTime() << " s." << endl << endl;

  for(int i=0;i<present->getNbValues();i++)
    {
    for(int j=0;j<present->getNbDimensions0();j++)
      {
      for(int k=0;k<present->getNbDimensions1();k++)
        {
        outputFile << present->getValue(i,j,k) << " ";
        }
      outputFile  << endl;
      }
    }
  outputFile  << flush;
  }

