// Copyright (C) 2004 by OpenFlower Team
// http://sourceforge.net/projects/openflower/
// Licensed under the GNU GPL Version 2.

#ifndef EQUATION_H
#define EQUATION_H

#include <stdlib.h>
#include <map>

#include "MyVector.h"

class Mesh;
class BoundaryCondition;
class Operator;
class Algorithm;
class Medium;
class Simulation;
class Variable;

class Equation {
public:
  Equation();
  virtual ~Equation(){};

  void associateMesh(Mesh * _pMesh){ pMesh = _pMesh;};
  void associateSimulation(Simulation * _pSimulation){ pSimulation = _pSimulation;};
  void associateMedium(Medium * _pMedium){ pMedium = _pMedium;};
  
  void read(ifstream& inputFile);
  virtual void readVirtual(ifstream& inputFile,string& keyword)=0;
  inline const MyVector<BoundaryCondition*> getRefBC()const {return boundaryConditions;};

  inline Variable& getPresent()const {return *present;};
  inline Variable& getPresent() {return *present;};
  inline Variable& getFuture()const {return *future;};
  inline Variable& getFuture() {return *future;};
  inline Operator& getOperator(const int i) {return *operators[i];};
  inline const Medium& getMedium()const {return *pMedium;};
  inline const Medium& getMedium() {return *pMedium;};

  inline const string& getLabel()const {return label;};
  inline const string& getLabel() {return label;};
  inline const string& getLabelofMedium()const {return labelOfMedium;};
  inline const string& getLabelofMedium() {return labelOfMedium;};
  inline const string& getNameOfEquation() {return name;};
  
  void advance();

  virtual void addsItsVariables4PostProcessing(map<string,const Variable*> & variablesMap)=0;
  virtual void finish(); //!< A set of callings to finish the simulation and save the data associated to the Equation.

protected:
  string name;          //!< The name of the Equation: Navier-Stokes, Scalar Diffusion, etc...
  string label;         //!< The label of this Equation used by an Algorithm.
  string labelOfMedium; //!< The label of the Medium attached to this Equation.
  Variable* present;    //!< The Variable containing the present values of the main unknown (specific to each Equation ).
  Variable* future;     //!< The Variable containing the future values of the main unknown which are to be calculated.
  int nbBC;             //!< The number of boundary-conditions applied to this Equation.
  int nbOp;             //!< The number of Mathematical operators defined for this Equation.
  MyVector<BoundaryCondition*> boundaryConditions; //!< A list of all the boundary-conditions associated to this Equation.
  MyVector<Operator*> operators; //!< A list of all the mathematical operators to be used by this Equation.
  const Mesh* pMesh;             //!< A pointer towards the Mesh of the Simulation.
  const Simulation* pSimulation; //!< A pointer towards the Simulation.
  const Medium* pMedium;         //!< A pointer towards the Medium to which the Equation is attached to.

  void initialize(Variable& variable, ifstream& inputFile); //!< Initializes the main unknown Variable.
  };

#endif
