// Copyright (C) 2004 by OpenFlower Team
// http://sourceforge.net/projects/openflower/
// Licensed under the GNU GPL Version 2.

#ifndef ELEMENT_H
#define ELEMENT_H

#include <MyMatrix.h>
 
using namespace std;

class Element {
public: 
  Element();
  ~Element();
  
  inline int getNbVertices() const { return nbVertices; };
  inline int getVertex(const int iv) const {assert(iv<this->nbVertices); return vertices[iv];};
  inline const  MyVector<double> & getXYZ() const { return XYZ; };
  inline double getVolume() const { return volume; };
  inline int getType() const { return type; };
  inline int getNbNeighbourFaces() const {return nbNeighbourFaces;};
  inline int getNeighbourFace(const int n) const {return neighbourFaces[n];};
  
  void setNbVertices(const int nv); 
  void setVertex(const int iv, const int num_v);
  void setType(const int _type) ;
  void setNbNeighbourFaces(const int n);
  void setNeighbourFace(const int n,const int num_n);

  void calculateProperties(const MyMatrix<double> & vertexXYZ);
  double calculateVolumeTetrahedron( const MyVector<double> & v1, const MyVector<double> & v2, const MyVector<double> & v3, const MyVector<double> & v4 );  

private:
  int nbVertices; //! Number of vertices composing the Element.
  MyVector<int> vertices; //! List of the numbers of the vertices composing the elements.
  MyVector<double> XYZ; //! Coordinates of the Element.
  MyVector<int> neighbourFaces; //! List of the numbers of the faces composing the elements.
  int nbNeighbourFaces; //! Number of faces composing the Element.
  double volume; //! Volume (in m3) of the Element.
  int type; //!< Type of element: tetrahedra 4, hexahedra 5, prisms 6, pyramids 7.
  void calculateVolume( const MyMatrix<double> & vertexXYZ);

};



/** GMSH convention:

 quadrangle:
                      1___2
                       |     |
                      4___3


tetrahedron:   3
                    | \
                    |   \
                    | _ \2
                   1\  /
                       4

hexahedron:
                     4______3
                     |\          | \
                     | 8______7
                     1| -------2 |
                      \|           \|
                       5______6



prism:
                3
                 |\
                 |  \
                 1-2
                  \   \
                   \    6
                     \  | \
                      \ |   \
                       4 -  5

pyramid:

                       4_____3
                        | \     / |
                        |    5   |
                        |  /    \ |
                       1_____2
**/

#endif

