// Copyright (C) 2004 by OpenFlower Team
// http://sourceforge.net/projects/openflower/
// Licensed under the GNU GPL Version 2.

#include"math.h"

#include"Face.h"



////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
Face::Face()
  { 
  XYZ.resize(3);
  surfaceVector.resize(3);
  unitSurfaceVector.resize(3);
  neighbourElements.resize(2);
  frontierNumber = -1;
  };



////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
Face::~Face()
  { 
  };



////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
void Face::setNbVertices(const int nv)
  {
  assert(nv>0);
  this->nbVertices = nv;
  vertices.resize(nv);
  }; 



////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
void Face::setVertex(const int iv, const int num_v)
  {
  assert(iv < this->nbVertices);
  vertices[iv] = num_v;
  };



////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
void Face::calculateProperties(const MyMatrix<double>& vertexXYZ)
  {
  int i;
  XYZ=vertexXYZ[vertices[0]];  // calculation of XYZ coordinates
  for (i=1;i<nbVertices;i++)
    {
    XYZ.add(vertexXYZ[vertices[i]]);
    }
  XYZ.multiply(1./nbVertices);

  double ax=vertexXYZ[vertices[1]][0]-vertexXYZ[vertices[0]][0];        // calculation of the normal vector
  double ay=vertexXYZ[vertices[1]][1]-vertexXYZ[vertices[0]][1];
  double az=vertexXYZ[vertices[1]][2]-vertexXYZ[vertices[0]][2];
  double bx=vertexXYZ[vertices[2]][0]-vertexXYZ[vertices[0]][0];
  double by=vertexXYZ[vertices[2]][1]-vertexXYZ[vertices[0]][1];
  double bz=vertexXYZ[vertices[2]][2]-vertexXYZ[vertices[0]][2];
  surfaceVector[0]=ay*bz-az*by;
  surfaceVector[1]=az*bx-ax*bz;
  surfaceVector[2]=ax*by-ay*bx;

  if (this->type==3) // for quadrangle in gmsh conventions (ie Vertices in order 0 1 2 3 )
    {
    ax=vertexXYZ[vertices[2]][0]-vertexXYZ[vertices[0]][0];        // calculation of the normal vector
    ay=vertexXYZ[vertices[2]][1]-vertexXYZ[vertices[0]][1];
    az=vertexXYZ[vertices[2]][2]-vertexXYZ[vertices[0]][2];
    bx=vertexXYZ[vertices[3]][0]-vertexXYZ[vertices[0]][0];
    by=vertexXYZ[vertices[3]][1]-vertexXYZ[vertices[0]][1];
    bz=vertexXYZ[vertices[3]][2]-vertexXYZ[vertices[0]][2];
    surfaceVector[0]+=ay*bz-az*by;
    surfaceVector[1]+=az*bx-ax*bz;
    surfaceVector[2]+=ax*by-ay*bx;
    }
  surfaceVector.multiply(0.5);

  // calculation of the surface
  surface=sqrt(surfaceVector[0]*surfaceVector[0]+surfaceVector[1]*surfaceVector[1]+surfaceVector[2]*surfaceVector[2]);
  
  // calculation of the unit surface vector, added to avoid a lot of un-necessary operations.
  unitSurfaceVector[0] = surfaceVector[0]/surface;
  unitSurfaceVector[1] = surfaceVector[1]/surface;
  unitSurfaceVector[2] = surfaceVector[2]/surface;
  };



////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
void Face::setFrontierNumber(const int f_num)
  {
  frontierNumber=f_num;
  };



////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
void Face::setType(const int _type)
  {
  assert( ((_type==2) || (_type==3)) );
  type=_type;
  };



////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
void Face::setNeighbourElement(const int in,const int num_n)
  {
  assert(((in==0) || (in==1)));
  neighbourElements[in]=num_n;
  };

