// Copyright (C) 2004 by OpenFlower Team
// http://sourceforge.net/projects/openflower/
// Licensed under the GNU GPL Version 2.

#ifndef FACE_H
#define FACE_H

#include <assert.h>

#include "MyMatrix.h"

using namespace std;

class Face
{
public:
  Face();
  ~Face();

  inline int getNbVertices() const { return nbVertices; };
  inline int getVertex(const int iv) const {assert(iv<this->nbVertices); return vertices[iv];};
  inline const MyVector<double> & getXYZ() const { return XYZ; };
  inline const MyVector<double> & getSurfaceVector() const { return surfaceVector; };
  inline const MyVector<double> & getUnitSurfaceVector() const { return unitSurfaceVector; };
  inline double getSurface() const { return surface; };
  inline int getFrontierNumber() const {return frontierNumber;};
  inline int getType() const { return type; };
  inline int getNeighbourElement(const int in) const {assert(((in==0) || (in==1))); return neighbourElements[in];};
  void setNbVertices(const int nv) ;
  void setVertex(const int iv, const int num_v);
  void setFrontierNumber(const int f_num);
  void setType(const int _type) ;
  void setNeighbourElement(const int in,const int num_n);
  void makeSurfaceVectorOpposite() {for (int i=0;i<3;i++) { surfaceVector[i]*=-1; unitSurfaceVector[i]*=-1;}};
  void calculateProperties( const MyMatrix<double> & vertexXYZ);

private:
  int nbVertices;
  int frontierNumber;     //!< Defines the frontier number
  int type;               //!< Type of face: 2=triangle, 3=quadrangle
  double surface;
  MyVector <int > vertices;
  MyVector<double> XYZ;
  MyVector<double> surfaceVector;
  MyVector<double> unitSurfaceVector;
  MyVector<int> neighbourElements;

};


#endif

