// Copyright (C) 2004 by OpenFlower Team
// http://sourceforge.net/projects/openflower/
// Licensed under the GNU GPL Version 2.

#ifndef MESH_H
#define MESH_H

#include <stdio.h>
#include <stdlib.h>
#include <fstream>

#include "MyMatrix.h"

using namespace std;

class Face;
class Element;
class Frontier;

class Octree;

class Mesh 
{
public: 
  Mesh();
  virtual ~Mesh();

  void read(ifstream& inputFile);    //!< Reads the mesh block characteristics in the data file.
  
  inline int getNbFaces() const {return nbFaces;};
  inline int getNbFacesFrontiers() const {return nbFacesFrontiers;};
  inline int getNbElements() const {return nbElements;};
  inline int getNbVertices() const {return nbVertices;};
  inline double getMinCaractLength() const {return minCaractLength;};

  inline const MyVector<Face> & getRefFaces() const {return faces;};
  inline const MyVector<Element> & getRefElements() const {return elements;};
  inline const MyVector<Frontier> & getRefFrontiers() const {return frontiers;};
  inline const MyMatrix<double> & getRefVerticesPos() const {return vertexXYZ;};
  inline MyVector<double> getMinXYZVertices() const {return MinXYZVertices;};
  inline MyVector<double> getMaxXYZVertices() const {return MaxXYZVertices;};

  inline Octree& myOctree() {return * octree;};

  inline void setFormat(string _format) {format = _format;};
  void writeMeshFlw(ifstream& dataFile) const ;   //!< Writes the mesh in a file in openflower format.
  virtual void writeMeshVirtual(ifstream& dataFile) const {};

  void checkMesh(); //!< Checks a mesh:volume integrity, correspondance between elements...

protected:
  string file;            //!< Name of the mesh file.
  string name;            //!< Name of the mesh domain.
  string format;          //!< Format of the mesh (default=gmsh).
  int nbVertices;         //!< Number of vertices.
  int nbEdges;            //!< Number of edges.
  int nbFaces;            //!< Number of faces (triangles 2 or quadrangles 3).
  int nbFacesFrontiers;   //!< Number of faces that corespond to all frontiers - all at the beginning of the Vector Faces.
  int nbElements;         //!< Number of elements (tetrahedra 4, pyramids 5, prisms 6 and hexahedra 7).
  int nbFrontiers;        //!< Number of frontiers.
  double minCaractLength; //!< The smallest characteristic length of the mesh to evaluate the stability time steps.
  
/* Vortexflow (03/09/2004)
Maybe it should be removed: useless ??! */
  MyMatrix<int> edges;    //!< Number of the two vertices composing each edge.
/* Vortexflow (03/09/2004)*/  
  MyMatrix<double> vertexXYZ;              //!< Contains (x,y,z) locations of all vertices in domain.
  MyVector<Face> faces;                    //!< List of all Faces in the domain.
  MyVector<Element> elements;              //!< List of all Elements in the domain.
  MyVector<Frontier> frontiers;            //!< List of Frontiers.
  MyVector<double> MinXYZVertices;
  MyVector<double> MaxXYZVertices;
  Octree* octree;

  virtual void readFrontiersVirtual(ifstream& InputFile){}; 
  virtual void readMeshVirtual(){};        //!< Reads a particular mesh format.

private:
  void calculateFacesElementsProperties();
  void calculateMinimalCharacteristicLength();


};

#endif
