// Copyright (C) 2004 by OpenFlower Team
// http://sourceforge.net/projects/openflower/
// Licensed under the GNU GPL Version 2.

#include <iostream>

#include "Octree.h"
#include "Mesh.h"
#include "MyMacros.h"
#include "Face.h"
#include "Element.h"

using namespace std;



void Octree::createNode(Node* node, const Node* parent, int i)
  {
  double xmin = parent->xmin;
  double ymin = parent->ymin;
  double zmin = parent->zmin;
  double xmax = parent->xmax;
  double ymax = parent->ymax;
  double zmax = parent->zmax;
  double xmid = parent->xmid;
  double ymid = parent->ymid;
  double zmid = parent->zmid;
  
  switch(i)
    {
    case Back_Bottom_Left:
      node->xmin = xmin;
      node->xmax = xmid;
      node->ymin = ymin;
      node->ymax = ymid;
      node->zmin = zmin;
      node->zmax = zmid;
      break;
  
    case Back_Top_Left:
      node->xmin = xmin;
      node->xmax = xmid;
      node->ymin = ymin;
      node->ymax = ymid;
      node->zmin = zmid;
      node->zmax = zmax;
      break;
    case Front_Bottom_Left:
      node->xmin = xmin;
      node->xmax = xmid;
      node->ymin = ymid;
      node->ymax = ymax;
      node->zmin = zmin;
      node->zmax = zmid;
      break;
    case Front_Top_Left:
      node->xmin = xmin;
      node->xmax = xmid;
      node->ymin = ymid;
      node->ymax = ymax;
      node->zmin = zmid;
      node->zmax = zmax;
      break;
    case Back_Bottom_Right:
      node->xmin = xmid;
      node->xmax = xmax;
      node->ymin = ymin;
      node->ymax = ymid;
      node->zmin = zmin;
      node->zmax = zmid;
      break;
    case Back_Top_Right:
      node->xmin = xmid;
      node->xmax = xmax;
      node->ymin = ymin;
      node->ymax = ymid;
      node->zmin = zmid;
      node->zmax = zmax;
      break;
    case Front_Bottom_Right:
      node->xmin = xmid;
      node->xmax = xmax;
      node->ymin = ymid;
      node->ymax = ymax;
      node->zmin = zmin;
      node->zmax = zmid;
      break;
    case Front_Top_Right:
      node->xmin = xmid;
      node->xmax = xmax;
      node->ymin = ymid;
      node->ymax = ymax;
      node->zmin = zmid;
      node->zmax = zmax;
      break;
    }
  node->xmid = (node->xmin+node->xmax)/2.;
  node->ymid = (node->ymin+node->ymax)/2.;
  node->zmid = (node->zmin+node->zmax)/2.;
  }



Node::Node()
  {
  nodes.resize(0);
  }

  
  
Node::~Node()
  {
  for(int i=0;i<nodes.size();i++) delete nodes[i];	
  nodes.resize(0);
  }


  
Octree::Octree()
  {
  }

  
  
Octree::~Octree()
  {
  delete root;
  }



void Octree::destroyNode(Node* node)
{
}



int Octree::dispatch(Node* node,double x, double y, double z)
  {
  
  double xmid = node->xmid;
  double ymid = node->ymid;
  double zmid = node->zmid;
  
  if(x<xmid)
    {
    if(y<ymid)
      {
      if(z<zmid) return Back_Bottom_Left;
      else       return Back_Top_Left;
      }
    else
      {
      if (z<zmid) return Front_Bottom_Left;
      else        return Front_Top_Left;
      }
    }
  else
    {
    if(y<ymid)
      {
      if(z<zmid) return Back_Bottom_Right;
      else       return Back_Top_Right;
      }
    else
      {
      if(z<zmid) return Front_Bottom_Right;
      else       return Front_Top_Right;
      }
    }
  
  ERROR_IN_FILE;
  cerr << "Error in Octree::dispatch"<<endl;
  exit(-1);
  }



void Octree::createRecursive(Node* node, MyMatrix<double>& Tab)
  {
  for(int i=0;i<8;i++)
    {
    node->nodes.resize(8);
    node->nodes[i] = new Node;
    createNode(node->nodes[i],node, i);
    }
    
  MyVector<int> ok;
  ok.resize(8);
  
  for(int i=0; i<8; i++) node->nodes[i]->nbElements=0;
  
  int number_of_the_node=0, j;
  for(int k=0; k<node->nbElements; k++)
    {
    for(unsigned int l=0;l<8;l++) ok[l]=0;
    int i = node->elements[k];
    double xm = Tab[i][0]-epsilon;
    double xp = Tab[i][0]+epsilon;
    double ym = Tab[i][1]-epsilon;
    double yp = Tab[i][1]+epsilon;
    double zm = Tab[i][2]-epsilon;
    double zp = Tab[i][2]+epsilon;
    number_of_the_node = dispatch(node,xm,ym,zm);
  //  cout << "number_of_the_node : " << number_of_the_node<<endl;
      if(ok[number_of_the_node]==0) ok[number_of_the_node]=1;
    number_of_the_node = dispatch(node,xm,ym,zp);
      if(ok[number_of_the_node]==0) ok[number_of_the_node]=1;
    number_of_the_node = dispatch(node,xm,yp,zm);
      if(ok[number_of_the_node]==0) ok[number_of_the_node]=1;
    number_of_the_node = dispatch(node,xm,yp,zp);
      if(ok[number_of_the_node]==0) ok[number_of_the_node]=1;
    number_of_the_node = dispatch(node,xp,ym,zm);
      if(ok[number_of_the_node]==0) ok[number_of_the_node]=1;
    number_of_the_node = dispatch(node,xp,ym,zp);
      if(ok[number_of_the_node]==0) ok[number_of_the_node]=1;
    number_of_the_node = dispatch(node,xp,yp,zm);
      if(ok[number_of_the_node]==0) ok[number_of_the_node]=1;
    number_of_the_node = dispatch(node,xp,yp,zp);
      if(ok[number_of_the_node]==0) ok[number_of_the_node]=1;
  
    for(unsigned int m=0;m<8;m++)
      if(ok[m]==1)
        {
        node->nodes[m]->nbElements++;
        j = node->nodes[m]->nbElements;
        node->nodes[m]->elements.resize(j);
        node->nodes[m]->elements[j-1] = i;
        }
    }
  
  for(int i=0;i<8;i++)
    {
    if(node->nodes[i]->nbElements>max_elems)  /** Then we need to construct another node starting from this one */
      {
      createRecursive(node->nodes[i], Tab);
      }
    else if(node->nodes[i]->nbElements!=0)  addLeaf(node->nodes[i]);
    }
  
  }



void Octree::searchDoubleFaces(MyVector<int>& result, const MyVector<Face>& faces, const vector<int>& sumv)
  {
  int face,another_face;
  int nbv1;
  int nbv2;
  int m;
  int ok1,count_ok;
  int nb;
  for(int i=0;i<nb_leafs;i++)
    {
    nb=leafs[i]->nbElements;
    //  cout << "Number of faces in leaf " << i << ": " << nb<<endl;  
    // A LOOP ON THE DIFFERENT NODES
    if((nb%2)!=0)
      {
      ERROR_IN_FILE;
      cerr << "Problem in octree: possibly two identical faces in two different leafs..."<<endl;
      cerr << "Send bug report at http://sourceforge.net/projects/openflower"<< endl;
      assert(0);
      }
    // For each node, we find the two corresponding double faces
    for(int j=0;j<nb/2;j++)
      {
      m=j;
      face=leafs[i]->elements[j];
      nbv1 = faces[face].getNbVertices();
      do
        {
        m++;
        another_face = leafs[i]->elements[m];
        nbv2 = faces[another_face].getNbVertices();
        ok1=1;
        count_ok=0;
        assert(m<nb-j);
        if((nbv1==nbv2)&&(sumv[face]==sumv[another_face]))
          {
          for(int k=0;k<3;k++) // Checking of nodes correspondance can be done only on 3 nodes
          // (the fourth node is not necessary)
            {
            for(int l=0;l<nbv2;l++)
              {
              if(faces[face].getVertex(k)==faces[another_face].getVertex(l)) count_ok++;
              }
            if(count_ok!=(k+1)) break;
            }
          if(count_ok==3) ok1=0;
          }
        else ok1=1;
        }
      while(ok1); // End of while loop
      
      result[face]=another_face;
      result[another_face]=face;
      leafs[i]->elements[m]=leafs[i]->elements[nb-j-1];
      }
    }
  }



void Octree::addLeaf(Node* node)
  {
  nb_leafs++;
  leafs.resize(nb_leafs);
  leafs[nb_leafs-1] = node;
  }



void Octree::createOctree(MyVector<double> min, MyVector<double> max, MyMatrix<double>& Tab, int nbElements)
  {
  nb_leafs = 0;
  
  min[0]=min[0]-2.*epsilon;
  max[0]=max[0]+2.*epsilon;
  min[1]=min[1]-2.*epsilon;
  max[1]=max[1]+2.*epsilon;
  min[2]=min[2]-2.*epsilon;
  max[2]=max[2]+2.*epsilon;
  
  root = new Node;
  root->xmin=min[0];
  root->xmax=max[0];
  root->ymin=min[1];
  root->ymax=max[1];
  root->zmin=min[2];
  root->zmax=max[2];
  
  root->xmid=(min[0]+max[0])/2.;
  root->ymid=(min[1]+max[1])/2.;
  root->zmid=(min[2]+max[2])/2.;
  
  root->elements.resize(nbElements);
  root->nbElements = nbElements;
  
  for(int i=0; i<nbElements; i++)
    {
    root->elements[i]=i;
    }
  createRecursive(root, Tab);
  
  //cout << "nb_leafs = " << nb_leafs << endl;
  }



void Octree::searchElementWithCoordinates(MyVector<double>& position, int& elementNumber, MyVector<double>& elementPosition)
  {
  double x = position[0];
  double y = position[1];
  double z = position[2];
  
// FIXXME (Vortexflow, 05 oct 2004) -> Now, the element containing the point at position "position"
// is determined by the smallest distance between an element and this position.
// Here we should call a method like: elements[anElement].containsPoint(position);
// which tells us if the point is situated inside the element.
  double smallestDistance = 1e30;
// END FIXXME
  MyVector<double> anElementPosition;
  anElementPosition.resize(3);
  int numberOfTheNode;
  int notFound = 1;
  int anElement;
  const MyVector<Element>& elements = pMesh->getRefElements();
  
  Node* nextNode;
  Node* currentNode;
  currentNode = root;
  
  while(notFound)
    {
    numberOfTheNode = dispatch(currentNode,x,y,z);
    nextNode = currentNode->nodes[numberOfTheNode];
    currentNode = nextNode;
    if(nextNode->nbElements < max_elems) notFound=0;
    }
          
  for(int k=0; k<currentNode->nbElements; k++)
    {
    anElement = currentNode->elements[k];
    anElementPosition = elements[anElement].getXYZ();
    if((pow(position[0]-anElementPosition[0],2.)+pow(position[1]-anElementPosition[1],2.)+pow(position[2]-anElementPosition[2],2.))<smallestDistance)
      {
      smallestDistance = pow(position[0]-anElementPosition[0],2.)+pow(position[1]-anElementPosition[1],2.)+pow(position[2]-anElementPosition[2],2.);
      elementNumber = anElement;
      }
    }
  elementPosition = elements[elementNumber].getXYZ();
  cout << "Element with position ("<<x<<","<<y<<","<<z<<") is situated inside/near element(";
  cout << elementNumber <<") located at ("<<elementPosition[0]<<","<<elementPosition[1]<<","<<elementPosition[2]<<")"<<endl;
  }
  
