// Copyright (C) 2004 by OpenFlower Team
// http://sourceforge.net/projects/openflower/
// Licensed under the GNU GPL Version 2.

#ifndef OCTREE_H
#define OCTREE_H


#include "MyMatrix.h"

#define Back_Bottom_Left   0
#define Back_Bottom_Right  1
#define Front_Bottom_Left  2
#define Front_Bottom_Right 3
#define Back_Top_Left      4
#define Back_Top_Right     5
#define Front_Top_Left     6
#define Front_Top_Right    7

#define epsilon   1e-6
#define max_elems 5000

using namespace std;

class Face;
class Element;
class Mesh;

class Node
{
 public :
  Node();
  ~Node();
  
  double xmin, xmax, ymin, ymax, zmin, zmax;
  double xmid, ymid, zmid;
  
  vector<Node*> nodes;
  MyVector<int> elements;
  int nbElements;
};


class Octree
{
 public:
  Octree();
  ~Octree();

  void createOctree(MyVector<double> min, MyVector<double> max, MyMatrix<double>& Tab, int nbElements);

  void searchDoubleFaces(MyVector<int>& result, const MyVector<Face>& faces, const vector<int>& sumv);
  void searchElementWithCoordinates(MyVector<double>& position, int& elementNumber, MyVector<double>& elementPosition);
  
  inline void associateMesh(const Mesh* _pMesh) {pMesh = _pMesh;}
 private:
  void destroyNode(Node* );
  void destroy();
  void createNode(Node* ,const Node* node, int i);
  void createRecursive(Node* , MyMatrix<double>& Tab);
  void addLeaf(Node* node);
  int  dispatch(Node* ,double x, double y, double z);
  
  Node* root;
  vector<Node*> leafs;
  int nb_leafs;
  const Mesh* pMesh;
};

#endif
