// Copyright (C) 2004 by OpenFlower
// http://sourceforge.net/projects/openflower/
// Licensed under the GNU GPL Version 2.

#include <fstream>
#include <iostream>

#include "Medium.h"
#include "Parser.h"
#include "MyMacros.h"

using namespace std;



////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
Medium::Medium()
  {
  mediumProperties.resize(4);
  mediumProperties[0].property = new Variable;
  mediumProperties[1].property = new Variable;
  mediumProperties[2].property = new Variable;
  mediumProperties[3].property = new Variable;
  
  mediumProperties[0].property->setName("rho");
  mediumProperties[1].property->setName("mu");
  mediumProperties[2].property->setName("lambda");
  mediumProperties[3].property->setName("Cp");
  }



////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
Medium::~Medium()
  {  
/*   for (int i=0;i<nbMediums;i++)
    for (int j=0; j<nbProperties;j++)
      {
      if (properties[i][j].isDefined==2)
        delete properties[i][j].expr;
      } */
  }



////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
void Medium::add(ifstream& inputFile)
  {  
  string keyword;

  inputFile >> keyword;
  label = keyword;
  
  inputFile >> keyword;
  if(keyword != "\{")
    {
    ERROR_IN_FILE;
    cerr << "Expecting an opening \"\{\" before Medium block!" << endl;
    exit(-1);
    }

  inputFile >> keyword;
  int i;
  while (keyword!="}")
    {
    if (keyword=="//")
      {
      getline(inputFile,keyword);
      }
    else
      {
      for(i=0;i<4;i++)
        {
        if(mediumProperties[i].property->getName()==keyword) break;
        }
      if(i==4)
        {
        ERROR_IN_FILE;
        cerr << keyword << " is an unknown property!" << endl;
        cerr << "  Known properties are:";
        for (int j=0;j<4;j++)
          {
          cerr << " " << mediumProperties[j].property->getName();
          }
        cerr << endl;
        exit(-1);
        }

      inputFile >> keyword;
      
      if(mediumProperties[i].isDefined)
        {
        INFORMATION_IN_FILE;
        cout << "WARNING: " << mediumProperties[i].property->getName() << " was already previously defined" << endl;
        cout << "with this expression: " << mediumProperties[i].expr->getExpression() << endl;
        cout << "Skipping this new definition of " << mediumProperties[i].property->getName() << endl;
        }
      else
        {
        mediumProperties[i].isDefined=1;
        mediumProperties[i].expr = new Parser;
        mediumProperties[i].expr->makeTree(keyword);
        }
      } // } of else
    inputFile >> keyword;
    } // } of while

  }


  
void Medium::setNbValuesOfProperties(int nbValues)
  {
  for(int i=0;i<4;i++)
    {
    if(mediumProperties[i].isDefined)
      {
      mediumProperties[i].property->resize(nbValues);
      }
    }
  }



void Medium::updateProperties()
  {
  double result;
  for(int i=0;i<4;i++)
    {
    if(mediumProperties[i].isDefined)
      {
      for(int j=0;j<mediumProperties[i].property->getNbValues();j++)
        {
//      XYZElement = elements[i].getXYZ();

//      expression.setVariable("X",XYZElement[0]);
//      expression.setVariable("Y",XYZElement[1]);
//      expression.setVariable("Z",XYZElement[2]);
        mediumProperties[i].expr->evaluate(result);
        mediumProperties[i].property->setValue(result,j);
        }
      }
    }
  }



void Medium::addsItsVariables4PostProcessing(map<string,const Variable*> & variablesMap)
  {
  string labelOfVariable;

  labelOfVariable=label;
  labelOfVariable+=".Rho";
  variablesMap[labelOfVariable]= mediumProperties[0].property;
  
  labelOfVariable=label;
  labelOfVariable+=".Mu";
  variablesMap[labelOfVariable]= mediumProperties[1].property;

  labelOfVariable=label;
  labelOfVariable+=".Lambda";
  variablesMap[labelOfVariable]= mediumProperties[2].property;

  labelOfVariable=label;
  labelOfVariable+=".Cp";
  variablesMap[labelOfVariable]= mediumProperties[3].property;
  }


////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
/*void Medium::getValue(const string& mediumName, const string& propertyName, const double& temperature , double& result)
  {
  int posM,posP;
	
  posM=mediumsNames.search(mediumName);
	
  if (posM==-1)
    {
    ERROR_IN_FILE;
    cerr << mediumName << " is an unknown Medium name!" << endl;
    cerr << "  Known mediums are: ";
    for (unsigned int i=0;i<mediumsNames.size();i++)
      {
      cerr << mediumsNames[i] << " ";
      }
    cerr << endl;
    exit(-1);
    }
	
  posP=propertiesNames.search(propertyName);

  if (posP==-1)
    {
    ERROR_IN_FILE;
    cerr << propertyName << " is an unknown property name!" << endl;
    cerr << "  Known properties are: ";
    for (unsigned int i=0;i<propertiesNames.size();i++)
      {
      cerr << propertiesNames[i] << " ";
      }
    cerr << endl;
    exit(-1);
    }

  switch (properties[posM][posP].isDefined)
    {
    case 0:
    {
    cerr << "Property " << propertyName << "is not defined for medium " << mediumName << endl;
    exit(-1);
    break;
    }
    case 1:
    {
    result=properties[posM][posP].value;
    break;
    }
    case 2:
    {
    properties[posM][posP].expr->setVariable("T",temperature);
    properties[posM][posP].expr->evaluate(result);
    break;
    }
    default: break;
    }

  };
*/
