// Copyright (C) 2004 by OpenFlower Team
// http://sourceforge.net/projects/openflower/
// Licensed under the GNU GPL Version 2.

#ifndef SIMULATION_H
#define SIMULATION_H

#include <stdlib.h>
#include <stdio.h>
#include <string>
#include <map>

#include "MyVector.h"

using namespace std;

class Algorithm;
class Equation;
class Medium;
class Mesh;
class PostProcessing;
class Variable;

class Simulation {
public:
  Simulation();
  ~Simulation();

  Mesh* pMesh;              //!< Mesh of the Simulation.
  inline int getNbEqn()const {return nbEqn;} ; //!< Returns the number (>=1) of Equations of the Simulation.
  inline const Equation* getRefEqn(int i)const {return pEqn[i];} ; //!< Returns a vector of pointers of Equations of the Simulation.

  inline double getTimeStep() {return timeStep;} ; //!< Returns the value of the time step of the Simulation.
  inline double getTime() const {return time;} ; //!< Returns the value of the current time of the Simulation.
  
  const Variable* getVariableFromMap(string& label);

  void setFile(const string _file) { file = _file; }; //!< Initializes the name of the data file.
  void read();                              //!< Reads information from FLW file and prepares computation.
  void prepare();                           //!< Prepares and checks all the attributes and sub-attributes of the Simulation.
  void solve();                             //!< Solves the CFD Simulation.

private: 
  MyVector<Equation*> pEqn; //!< The list of all Equations present in the simulation.
  MyVector<Algorithm*> pAlgorithms; //!< The list of all the Algorithms defined in the Simulation.
  MyVector<Medium*> pMediums; //!< The list of all the Mediums involved in the Simulation.
  PostProcessing* PostProc; //!< Post-processing associated to the Simulation.
  typedef map<string,const Variable*> NameAndVariablePointerMap;
  NameAndVariablePointerMap mapNameVariable; // Map containing the correspondance between a Variable and its identified name.

  string file;              //!< Name of the data file containing the definition of the Simulation.
  int nbEqn;                //!< The number (>=1) of Equations of the Simulation.
  int nbAlgo;               //!< The number of Algorithms defined in the Simulation.
  int nbMediums;            //!< The number of Mediums defined in the Simulation.
  
  double timeBegin;
  double timeEnd;
  double time;
  double timeStep;
  int nbOfTimeStep;
  int maximumTimeSteps;
  double cfl;

  
  void readMesh(ifstream& inputFile);       //!< Reads the Mesh Block from the FLW file.
  void addEqn(ifstream& inputFile);         //!< Adds an Equation to solve in the Simulation.
  void addAlgorithm(ifstream& inputFile);
  bool simulationIsNotOver();
  void calculateTimeStep();                 //!< Gets several stability time steps and keeps lowest.
  void finishSimulation();

  void associateVariables2PostProcessings();
  void initializesProbes2PosProcessings();

};

#endif
