// Copyright (C) 2004 by OpenFlower Team
// http://sourceforge.net/projects/openflower/
// Licensed under the GNU GPL Version 2.

#include <iostream>
#include <fstream>
#include <sstream>

#include "PostGlobal.h"
#include "MyMacros.h"
#include "PostProcField.h"
#include "PostProcFieldGmsh.h"
#include "MyVector.h"
//#include "Mesh.h"
//#include "Variable.h"

using namespace std;

class Mesh;
class Variable;

PostGlobal::PostGlobal()
  {
  lastPostProcessingTime = -1000.;
  mainFile = new ofstream;
  }



PostGlobal::~PostGlobal()
  {
  }



// Reads in the input file, the block related to PostGlobal definition
void PostGlobal::read(ifstream& inputFile)
  {
  MyVector<string> knownKeywords;
  knownKeywords.resize(1);
  knownKeywords[0]="gmsh";
  string keyword;

  inputFile >> keyword;

  switch(knownKeywords.search(keyword))
    {
    case 0:
          {
          pPostProcField = new PostProcFieldGmsh;
          pPostProcField->setMesh(pMesh);
          break;
          }
    case -1:
           {
           ERROR_IN_FILE;
           cerr << keyword << " is an unknown output file format for PostGlobal block!" << endl;
           cerr << "  Known keywords are: ";
           for (unsigned int i=1;i<knownKeywords.size();i++)
             {
             cerr << knownKeywords[i] << " ";
             }
           cerr << endl;
           exit(-1);
           break;
           }
    default: break;  // impossible case
    }
    
  inputFile >> fieldName;
  inputFile >> tInit;
  if(inputFile.fail())
    {
    ERROR_IN_FILE;
    cerr << "A numerical value was expected for tInit " << endl;
    exit(-1);
    }
  inputFile >> tEnd;
  if(inputFile.fail())
    {
    ERROR_IN_FILE;
    cerr << "A numerical value was expected for tEnd " << endl;
    exit(-1);
    }
  inputFile >> dt;
  if(inputFile.fail())
    {
    ERROR_IN_FILE;
    cerr << "A numerical value was expected for dt " << endl;
    exit(-1);
    }
  }

  
  
void PostGlobal::createOutputFile()
  {
  // Here, by default we erase what is contained in the file if it exists with the option ios::trunc
  // If we decide to append the data if the file exists, we should replace by ios::app
  // FIXXME (Vortexflow, 7 oct 2004) -> Here we assume that Post-Processing is only GMSH!!!!
  string sFileName;
  sFileName=fieldName;
  sFileName+=".pos";
  // END FIXXME

  mainFile->open(sFileName.c_str(), ios::out | ios::trunc);

  if(mainFile->bad())
    {
    ERROR_IN_FILE;
    cerr << "Error in opening/creating Main Global PostProcessing output file: " << sFileName << endl;
    exit(-1);
    }
  // Here we should probably write some information in the header of the file...
  }

  
  
void PostGlobal::execute(double time, bool forceExecution)
  {
  if(((time>=(lastPostProcessingTime+dt))&&(time<=tEnd)&&(time>=tInit)) || (forceExecution==1))
    {
    string sTime;
    stringstream stream;
    stream << time;
    stream >> sTime;

  // FIXXME (Vortexflow, 7 oct 2004) -> Here we assume that Post-Processing is only GMSH!!!!
    string fileName;
    fileName=fieldName;
    fileName+="_";
    fileName+=sTime;
    fileName+=".pos";
    
    pPostProcField->PostProcVariable(time,fileName,(*Var));
    lastPostProcessingTime = time;
    *mainFile << "Merge \"" << fileName << "\" ;" << endl << flush;
  // END FIXXME
    }

  }

  
  
void PostGlobal::finish()
  {
  // FIXXME (Vortexflow, 7 oct 2004) -> Here we assume that Post-Processing is only GMSH!!!!
    *mainFile << "Combine TimeSteps;" << endl;
    *mainFile << "For num In {0:PostProcessing.NbViews-2}" << endl;
    *mainFile << " Delete View[0];" << endl;
    *mainFile << "EndFor" << endl << flush;
  // END FIXXME
    mainFile->close();
  }
