// Copyright (C) 2004 by OpenFlower Team
// http://sourceforge.net/projects/openflower/
// Licensed under the GNU GPL Version 2.

#include <iostream>
#include <fstream>
#include <sstream>

#include "PostProbe.h"
#include "MyMacros.h"
#include "Variable.h"

using namespace std;

PostProbe::PostProbe()
  {
  probePosition.resize(3);
  lastPostProcessingTime = -1000.;
  file = new ofstream;
  }



PostProbe::~PostProbe()
  {
  }



// Reads in the inputFile, the block related to PostProbe definition
void PostProbe::read(ifstream& inputFile)
  {
  string keyword;
  
  inputFile >> fieldName;
  inputFile >> tInit;
  if(inputFile.fail())
    {
    ERROR_IN_FILE;
    cerr << "A numerical value was expected for tInit" <<endl;
    exit(-1);
    }
  inputFile >> tEnd;
  if(inputFile.fail())
    {
    ERROR_IN_FILE;
    cerr << "A numerical value was expected for tEnd" << endl;
    exit(-1);
    }
  inputFile >> dt;
  if(inputFile.fail())
    {
    ERROR_IN_FILE;
    cerr << "A numerical value was expected for dt" << endl;
    exit(-1);
    }
  inputFile >> keyword;
  if(keyword=="position")
    {
    inputFile >> probePosition[0];
    inputFile >> probePosition[1];
    inputFile >> probePosition[2];
    }
  else
    {
    ERROR_IN_FILE;
    cerr << "Expecting keyword \"position\" to introduce the x,y,z coordinates of the probe location..." << endl;
    exit(-1);
    }
    
  }

  
  
void PostProbe::createOutputFile(int numberOfTheProbe)
  {
  // Here, by default we erase what is contained in the file if it exists with the option ios::trunc
  // If we decide to append the data if the file exists, we should replace by ios::app
  string sNb;
  string sBase;
  sBase="Probe_";
  {
  stringstream stream;
  stream << numberOfTheProbe;
  stream >> sNb;
  }
  sBase+=sNb;
  sBase+="_";
  sBase+=fieldName;
  sBase+=".probe";
  
  file->open(sBase.c_str(), ios::out | ios::trunc);

  if(file->bad())
    {
    ERROR_IN_FILE;
    cerr << "Error in opening/creating PostProbe output file: " << sBase << endl;
    exit(-1);
    }
  time_t t;
  time(&t);
  *file << "# Generated by OpenFlower CFD Software on " << ctime(&t);
  *file << "# Probe File on Variable: " << fieldName << endl;
  *file << "# Stituated at point (x=" << probePosition[0] << " ,y="
        << probePosition[1] << " ,z=" << probePosition[2] << ")" << endl;
  *file << "# Time   Field-value" << endl;
  }



void PostProbe::execute(double time, bool forceExecute)
  {
  if(((time>=(lastPostProcessingTime+dt))&&(time<=tEnd)&&(time>=tInit)) || (forceExecute == 1))
    {
    *file << time;
    for(int i=0;i<var->getNbDimensions0();i++)
      {
      for(int j=0;j<var->getNbDimensions1();j++)
        {
        *file << " " << var->getValue(elementNumber,i,j);
        }
      }
    *file << endl << flush;
    lastPostProcessingTime = time;
    }
  }
