// Copyright (C) 2004 by OpenFlower Team
// http://sourceforge.net/projects/openflower/
// Licensed under the GNU GPL Version 2.

#include <iostream>
#include <fstream>

#include "PostProcFieldGmsh.h"
#include "MyMatrix.h"
#include "Variable.h"
#include "Mesh.h"
#include "Element.h"

using namespace std;

////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
PostProcFieldGmsh::PostProcFieldGmsh()
  {
  }

  
  
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
PostProcFieldGmsh::~PostProcFieldGmsh()
  {
  }



////////////////////////////////////////////////////////////
// Post-processing method for a vector variable Field in GMSH format
////////////////////////////////////////////////////////////
void PostProcFieldGmsh::PostProcVariable(double time, string fileName, const Variable& field)
  {

  unsigned int i, j, k, m;
  int nbDims0 = field.getNbDimensions0();
  int nbDims1 = field.getNbDimensions1();
  double val=0;

  vector<int> countElements(7);
  for(i=0;i<7;i++)
    {
    countElements[i] = 0;
    }

  ofstream outputFile(fileName.c_str());
  string view_name =field.getName();
  
  int nb_time_steps=1,
  nbScalarPoints=0      , nbVectorPoints=0       , nbTensorPoints      =0,
  nbScalarLines=0       , nbVectorLines=0        , nbTensorLines       =0,
  nbScalarTriangles=0   , nbVectorTriangles=0    , nbTensorTriangles   =0,
  nbScalarQuadrangles=0 , nbVectorQuadrangles=0  , nbTensorQuadrangles =0,
  nbScalarTetrahedra=0  , nbVectorTetrahedra=0   , nbTensorTetrahedra  =0,
  nbScalarHexahedra=0   , nbVectorHexahedra=0    , nbTensorHexahedra   =0,
  nbScalarPrisms=0      , nbVectorPrisms=0       , nbTensorPrisms      =0,
  nbScalarPyramids=0    , nbVectorPyramids=0     , nbTensorPyramids    =0,
  nbText2d=0            , nbText2d_chars=0,
  nbText3d=0            , nbText3d_chars=0;


  const int nbElements = pMesh->getNbElements();
  const int nbVertices = pMesh->getNbVertices();
  const MyVector<Element> & elements = pMesh->getRefElements();
  const MyMatrix<double> & vertex_xyz = pMesh->getRefVerticesPos();

  Variable fieldOnVertices;
  fieldOnVertices.resize(nbVertices,nbDims0*nbDims1);
  vector<double> totalVolumeAroundVertex(nbVertices);
  for(i=0;i<nbVertices;i++)
    {
    totalVolumeAroundVertex[i] = 0.;
    }
  
  for(i=0;i<nbElements;i++)
    {
    countElements[elements[i].getType()-1]++;
    for(j=0;j<elements[i].getNbVertices();j++)
      {
      for(k=0;k<nbDims0;k++)
        {
        for(m=0;m<nbDims1;m++)
          {
          fieldOnVertices.addValue(field.getValue(i,k,m)*elements[i].getVolume(), elements[i].getVertex(j), k*nbDims1+m);
          }
        }
      totalVolumeAroundVertex[elements[i].getVertex(j)]+=elements[i].getVolume();
      }
    }
  for(i=0;i<nbVertices;i++)
    {
    for(k=0;k<nbDims0*nbDims1;k++)
      {
      val = fieldOnVertices.getValue(i, k);
      fieldOnVertices.setValue(val/totalVolumeAroundVertex[i], i, k);
      }
    }


  if((field.getLocation()=="cell")&&(nbDims0==1)&&(nbDims1==1)) // It's a scalar field
    {
    nbScalarTetrahedra = countElements[3];
    nbScalarHexahedra  = countElements[4];
    nbScalarPrisms     = countElements[5];
    nbScalarPyramids   = countElements[6];
    }
  else if((field.getLocation()=="cell")&&((nbDims0*nbDims1)==3)) // It's a vectorial field
    {
    nbVectorTetrahedra = countElements[3];
    nbVectorHexahedra  = countElements[4];
    nbVectorPrisms     = countElements[5];
    nbVectorPyramids   = countElements[6];
    }
  else // It's a tensorial field
    {
    nbTensorTetrahedra = countElements[3];
    nbTensorHexahedra  = countElements[4];
    nbTensorPrisms     = countElements[5];
    nbTensorPyramids   = countElements[6];
    }


  outputFile << "$PostFormat\n1.2 0 "<<sizeof(double)<<endl;
  outputFile << "$EndPostFormat\n$View\n";
  outputFile << view_name              << " " << nb_time_steps<<endl;
  outputFile << nbScalarPoints         << " " << nbVectorPoints      << " " << nbTensorPoints << endl;
  outputFile << nbScalarLines          << " " << nbVectorLines       << " " << nbTensorLines << endl;
  outputFile << nbScalarTriangles      << " " << nbVectorTriangles   << " " << nbTensorTriangles << endl;
  outputFile << nbScalarQuadrangles    << " " << nbVectorQuadrangles << " " << nbTensorQuadrangles << endl;
  outputFile << nbScalarTetrahedra     << " " << nbVectorTetrahedra  << " " << nbTensorTetrahedra << endl;
  outputFile << nbScalarHexahedra      << " " << nbVectorHexahedra   << " " << nbTensorHexahedra << endl;
  outputFile << nbScalarPrisms         << " " << nbVectorPrisms      << " " << nbTensorPrisms << endl;
  outputFile << nbScalarPyramids       << " " << nbVectorPyramids    << " " << nbTensorPyramids << endl;
  outputFile << nbText2d << " " << nbText2d_chars << " " << nbText3d << " " << nbText3d_chars << endl << endl;
  outputFile << time << endl;

  for(i=0; i<nbElements;i++)
    {
    for(k=0;k<3;k++)
      {
      for(j=0;j<elements[i].getNbVertices();j++)
        {
        outputFile << vertex_xyz[elements[i].getVertex(j)][k]<<" ";
        }
      }
    for(j=0;j<elements[i].getNbVertices();j++)
      {
      for(m=0;m<nbDims0*nbDims1;m++)
        {
        outputFile << fieldOnVertices.getValue(elements[i].getVertex(j),m)<<" ";
        }
      }
    outputFile << endl;
    }
  
  outputFile << "\n$EndView\n"<< endl << flush;
  outputFile.close();
  
  };
