// Copyright (C) 2004 by OpenFlower Team
// http://sourceforge.net/projects/openflower/
// Licensed under the GNU GPL Version 2.

#include <iostream>
#include <fstream>

#include "PostProcessing.h"

PostProcessing::PostProcessing()
  {
  nbPostProbe = 0;
  nbPostGlobal = 0;
  }

  
  
  PostProcessing::~PostProcessing()
  {
  }

  
  
// Reads Post-processing block in input file.
void PostProcessing::read(ifstream& inputFile)
  {
  string keyword;
  inputFile >> keyword;
  if(keyword != "\{")
    {
    cerr << "Expecting an opening \"\{\" before Post-processing block!" << endl;
    exit(-1);
    }
  MyVector<string> knownKeywords;
  knownKeywords.resize(3);
  knownKeywords[0]="//";
  knownKeywords[1]="probe";
  knownKeywords[2]="global";

  inputFile >> keyword;

  while (keyword!="}")
    {
    switch(knownKeywords.search(keyword))
      {
      case 0: getline(inputFile,keyword);  break;
      case 1: {
              nbPostProbe++;
              postProbeList.resize(nbPostProbe);
              postProbeList[nbPostProbe-1].read(inputFile);
              break;
              }
      case 2: {
              nbPostGlobal++;
              postGlobalList.resize(nbPostGlobal);
              postGlobalList[nbPostGlobal-1].setMesh(pMesh);
              postGlobalList[nbPostGlobal-1].read(inputFile);
              break;
              }
      case -1:
              {
              cerr << keyword << " is an unknown keyword for Post-processing block!" << endl;
              cerr << "  Known keywords are: ";
              for (unsigned int i=1;i<knownKeywords.size();i++)
                cerr << knownKeywords[i] << " ";
              cerr << endl;
              exit(-1);
              break;
              }
      default: break;  // impossible case
      }
    inputFile >> keyword;
    }
  cout << "Post-processing:"<<endl;
  cout << "  * Number of probes: "<<nbPostProbe<<endl;  
  cout << "  * Number of global posts: "<<nbPostGlobal<<endl;  

  }



void PostProcessing::execute(double time)
  {
  for(int i=0;i<nbPostGlobal;i++)
    {
    postGlobalList[i].execute(time);
    }

  for(int i=0;i<nbPostProbe;i++)
    {
    postProbeList[i].execute(time);
    }
  }



void PostProcessing::finish(double time)
  {
  for(int i=0;i<nbPostGlobal;i++)
    {
    postGlobalList[i].execute(time,1);
    postGlobalList[i].finish();
    }

  for(int i=0;i<nbPostProbe;i++)
    {
    postProbeList[i].execute(time,1);
    postProbeList[i].finish();
    }
  }

