// Copyright (C) 2004 by OpenFlower Team
// http://sourceforge.net/projects/openflower/
// Licensed under the GNU GPL Version 2.

#include "TimeScheme.h"
#include "Simulation.h"
#include "MyMacros.h"

////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
TimeScheme::TimeScheme()
  {
  
  };


////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
void TimeScheme::read(ifstream& inputFile)
  {
  string keyword; 

  inputFile >> keyword;
  if(keyword != "\{")
    {
    ERROR_IN_FILE;
    cerr << "Expecting an opening \"\{\" before Time Scheme block!" << endl;
    assert(0);
    exit(-1);
    }
  inputFile >> keyword;

  MyVector<string> knownKeywords;
  knownKeywords.resize(3);
  knownKeywords[0]="//";
  knownKeywords[1]="TimeBegin";
  knownKeywords[2]="TimeEnd";
	
  while (keyword!="}")
    {
    switch(knownKeywords.search(keyword))
      {
      case  0 : getline(inputFile,keyword); break;
      case  1 :
              {
	      inputFile >> timeBegin;
	      cout << "Initial Time : "<<timeBegin<<endl;
              break;
              }
      case  2 :
              {
	      inputFile >> timeEnd;
	      cout << "Final Time   : "<<timeEnd<<endl;
              break;
              }
      case -1 : //this->ReadVirtual(input_file,Keyword);
               break;
      default : break ;
      }
      inputFile >> keyword;
    }
  };



/*!\brief Makes a time step for the whole simulation.
 *
 * This method calls the MakeTimeStepOnEquation() function
 * for each equation present in the simulation and makes time advancement
 * for all of them.
 *
 */
void TimeScheme::makeTimeStep()
  {
  int nbEquations = pSimulation->getNbEqn();
  int i=0;
  for(i=0;i<nbEquations;i++)
    {
    const Equation* pEqn = pSimulation->getRefEqn(i);
    makeTimeStepOnEquation(pEqn);
    }
  }



