// Copyright (C) 2004 by OpenFlower Team
// http://sourceforge.net/projects/openflower/
// Licensed under the GNU GPL Version 2.


#include <algorithm>
#include <sstream>
#include <iostream>
#include <math.h>

#include "Parser.h"
#include "MyMacros.h"

using namespace std;

const string Parser::OPERATIONS="-+/*<>^"; // list of existing operations (note that the position of each operation in the string OPERATION is used later to determine the operation employed)
const string Parser::PARENTHESIS="()";



//////////////////////////////////////////////////////
//////////////////////////////////////////////////////
Parser::Parser() 
  {
  functions.push_back("MINUS");
  functions.push_back("COS");
  functions.push_back("SIN");
  functions.push_back("TAN");
  functions.push_back("ACOS");
  functions.push_back("ASIN");
  functions.push_back("ATAN");
  functions.push_back("COSH");
  functions.push_back("SINH");
  functions.push_back("TANH");
  functions.push_back("ACOSH");
  functions.push_back("ASINH");
  functions.push_back("ATANH");
  functions.push_back("EXP");
  functions.push_back("LOG");
  functions.push_back("LOG10");
  functions.push_back("SQRT");
  functions.push_back("RAND");
  srand(time(NULL));
  
  variables.push_back("PI");
  variables_value.push_back(4*atan(1.0));
  addVariable("X");
  addVariable("Y");
  addVariable("Z");
  
  } 
 
  
         
//////////////////////////////////////////////////////
//////////////////////////////////////////////////////
Parser::~Parser() 
  {
  }

  
  
//////////////////////////////////////////////////////
//////////////////////////////////////////////////////
void Parser::clear()
  {
  dq_op.resize(0);
  dq_ob.resize(0);
  formula_nodes.resize(0);
  variables.resize(0);
  variables_value.resize(0);
  variables.push_back("PI");
  variables_value.push_back(4*atan(1.0));
  }


  
//////////////////////////////////////////////////////
//////////////////////////////////////////////////////
void Parser::addVariable(string var)
  {
  upperCase(var);
  variables.push_back(var);
  variables_value.push_back(0.0);
  }

  
  
//////////////////////////////////////////////////////
//////////////////////////////////////////////////////
void Parser::setVariable(string _var, const double var_value)
  {
  string var;
  var=_var;
  upperCase(var);
  int pos;
  pos=stringExists(variables,var);
  if(pos!=-1)
    {
    variables_value[pos]=var_value;
    }
  else
    {
    ERROR_IN_FILE;
    cerr << var << " is an unknown variable in Mathematical Parser!" <<endl;
    exit(-1);
    }
  }

  
  
//////////////////////////////////////////////////////
//////////////////////////////////////////////////////
void Parser::makeTree(string formula)
  {
  expression = formula;
  formula+="+0";
  upperCase(formula);
  cutFormula(formula);

  int state=0;

//  for(int i=0;i<formula_nodes.size();i++)
//    {
//    cout << " i = " << i << " :  ";
//    cout << " " << formula_nodes[i].type ;
//    cout << " " << formula_nodes[i].expression ;
//    cout << " " << formula_nodes[i].value ;
//    cout << endl;
//    }

  for (int i=0;i<formula_nodes.size();i++)
    {
// 		cout << "step number " << i << endl;
// 		if (dq_op.size()!=0)
// 		{
// 			cout << " dq_op = "  ;
// 			cout << dq_op.back()->type << " " ;
// 			cout << dq_op.back()->expression << " " ;
// 			cout << dq_op.back()->value << " " ;
// 			cout << endl;
// 		}
// 		if (dq_ob.size()!=0)
// 		{
// 			cout << " dq_ob = "  ;
// 			cout << dq_ob.back()->type << " " ;
// 			cout << dq_ob.back()->expression << " " ;
// 			cout << dq_ob.back()->value << " " ;
// 			cout << endl;
// 		}

    switch (state)
      {
      case 0:		// initial state
        {
        makeTreeState0(i,state);
        break;
        }
      case 1:		// operande lecture
        {
        makeTreeState1(i,state);
        break;
        }
      case 2:		// operation lecture
        {
        makeTreeState2(i,state);
        break;
        }
      default:
        {
        ERROR_IN_FILE;
        cerr << "Problem while reading formula near " << formula_nodes[i].expression << endl;
        exit(-1);
        }
      }
    }
  while(dq_op.size()>0)
    {
    de_queued();
    }
  nodeRoot = dq_ob.back();
  }

  
  
//////////////////////////////////////////////////////
//////////////////////////////////////////////////////
void Parser::makeTreeState0(const int pos,int & state)
  {
  switch(formula_nodes[pos].type)
    {
    case 1: // OPERATION
    case 2: // PARENTHESIS
      {
      if(formula_nodes[pos].value==0) // ie "("
        {
        dq_op.push_back(&formula_nodes[pos]);
        state=0;
        break;
        }
      }
    case 3: // VALUE
      {
      dq_ob.push_back(&formula_nodes[pos]);
      state=2;			
      break;
      }
    case 4: // FUNCTION
      {
      dq_op.push_back(&formula_nodes[pos]);
      state=0;			
      break;
      }
    case 5: // VARIABLE
      {
      dq_ob.push_back(&formula_nodes[pos]);
      state=2;			
      break;
      }
    default:
      {
      ERROR_IN_FILE;
      cerr << "Problem while reading formula near " << formula_nodes[pos].expression ;
      cerr << " within state " << state << endl;
      exit(-1);
      }
    }
  }

  
  
//////////////////////////////////////////////////////
//////////////////////////////////////////////////////
void Parser::makeTreeState1(const int pos,int & state)
  {
  switch (formula_nodes[pos].type)
    {
    case 1: // OPERATION
    case 2: // PARENTHESIS
      {
      if(formula_nodes[pos].value==0) // ie "("
        {
        dq_op.push_back(&formula_nodes[pos]);
        state=0;
        break;
        }
      }
    case 3: // VALUE
      {
      dq_ob.push_back(&formula_nodes[pos]);
      state=2;			
      break;
      }
    case 4: // FUNCTION
      {
      dq_op.push_back(&formula_nodes[pos]);
      state=0;			
      break;
      }
    case 5: // VARIABLE
      {
      dq_ob.push_back(&formula_nodes[pos]);
      state=2;			
      break;
      }
    default:
      {
      ERROR_IN_FILE;
      cerr << "Problem while reading formula near " << formula_nodes[pos].expression ;
      cerr << " within state " << state << endl;
      exit(-1);
      }
    }
  }

  
  
//////////////////////////////////////////////////////
//////////////////////////////////////////////////////
void Parser::makeTreeState2(const int pos,int & state)
  {
  switch(formula_nodes[pos].type)
    {
    case 1: // OPERATION
      {
      while( ( (dq_op.size()!=0) && (dq_op.back()->value > (formula_nodes[pos].value-2)) ) && (dq_op.back()->type==ParserNode::OPERATION) )
        {
        de_queued();
        }
      dq_op.push_back(&formula_nodes[pos]);
      state=1;
      break;
      }
    case 2: // PARENTHESIS
      {
      if(formula_nodes[pos].value==1) // ie ")"
        {
        while(dq_op.back()->expression!="(")
          {
          de_queued();
          }
        dq_op.pop_back();	
        if(dq_op.size()!=0)
          {
          if(dq_op.back()->type==4) // ie a function
            {
            dq_op.back()->right = dq_ob.back();
            dq_ob.pop_back();
            dq_ob.push_back(dq_op.back());
            dq_op.pop_back();
            }
          }
        state=2;
        break;
        }
      }
    case 3: // VALUE
    case 4: // FUNCTION
    case 5: // VARIABLE
    default:
      {
      ERROR_IN_FILE;
      cerr << "Problem while reading formula near " << formula_nodes[pos].expression ;
      cerr << " within state " << state << endl;
      exit(-1);
      }
    }
  }

  
  
//////////////////////////////////////////////////////
//////////////////////////////////////////////////////
void Parser::de_queued()
  {
  dq_op.back()->right = dq_ob.back();
  dq_ob.pop_back();
  dq_op.back()->left = dq_ob.back();
  dq_ob.pop_back();
  dq_ob.push_back(dq_op.back());
  dq_op.pop_back();
  }

  
  
//////////////////////////////////////////////////////
//////////////////////////////////////////////////////
void Parser::cutFormula(const string& formula)
  {
  vector<int> cut_position;
  size_t pos;
  string part_formula;
  double d;
  stringstream stream;


//  cout << formula << endl;

  // search the operations positions in the formula
  for (int j=0; j<OPERATIONS.size(); j++)
    {
    pos=formula.find(OPERATIONS[j]);
    while (pos!=string::npos)
      {
      cut_position.push_back(pos);
      pos=formula.find(OPERATIONS[j],pos+1);
      }
    }

  // search the parenthesis positions in the formula
  for (int j=0; j<PARENTHESIS.size(); j++)
    {
    pos=formula.find(PARENTHESIS[j]);
    while (pos!=string::npos)
      {
      cut_position.push_back(pos);
      pos=formula.find(PARENTHESIS[j],pos+1);
      }
    }

  // sort the positions in right order and define the
  // positions of other than operations&parenthesis
  sort(cut_position.begin(), cut_position.end());
  pos=cut_position.size()-1;
  for (int i=0;i<pos;i++)
    {
    if ( cut_position[i+1] != (cut_position[i]+1) )
      {
      cut_position.push_back(cut_position[i]+1);
      }
    }
  if (cut_position[pos] != (formula.size()-1) )
    {
    cut_position.push_back(cut_position[pos]+1);
    }
  if (cut_position[0] != 0 )
    {
    cut_position.push_back(0);
    }
  cut_position.push_back(formula.size());
  sort(cut_position.begin(), cut_position.end());

  // make the vector of parser_node
  size_t s;
  for (int i=0;i<(cut_position.size()-1);i++)
    {
    s=formula_nodes.size();
    formula_nodes.resize(s+1);

    part_formula=formula.substr(cut_position[i],cut_position[i+1]-cut_position[i]);

    if ( PARENTHESIS.find(part_formula) !=string::npos )
      {
      formula_nodes[s].type=ParserNode::PARENTHESIS;
      formula_nodes[s].expression=part_formula;
      formula_nodes[s].value=PARENTHESIS.find(part_formula);
//  cout << "PARENTHESIS " << part_formula << endl;
      }
    else if ( OPERATIONS.find(part_formula) !=string::npos )
      {
      if (part_formula=="-")
        {
        if (s==0)
          {
          formula_nodes[s].type=ParserNode::VALUE;
          formula_nodes[s].expression="0.0";
          formula_nodes[s].value=0.0;
          formula_nodes[s].result=0.0;
          s+=1;
          formula_nodes.resize(s+1);
          }
        else if ( formula_nodes[s-1].expression=="(" )
          {
          formula_nodes[s].type=ParserNode::VALUE;
          formula_nodes[s].expression="0.0";
          formula_nodes[s].value=0.0;
          formula_nodes[s].result=0.0;
          s+=1;
          formula_nodes.resize(s+1);
          }
        }
      formula_nodes[s].type=ParserNode::OPERATION;
      formula_nodes[s].expression=part_formula;
      formula_nodes[s].value=OPERATIONS.find(part_formula);
//      cout << "OPERATION " << part_formula << endl;
      }
    else if ((part_formula[0] >= '0') && ( part_formula[0] <= '9') )
      {
      stream << part_formula << endl;
      stream >> d;
      formula_nodes[s].type=ParserNode::VALUE;
      formula_nodes[s].expression=part_formula;
      formula_nodes[s].value=d;
      formula_nodes[s].result=d;
//      cout << " double " << d << endl;
      }
    else if (stringExists(variables,part_formula)!=-1)
      {
      formula_nodes[s].type=ParserNode::VARIABLE;
      formula_nodes[s].expression=part_formula;
      formula_nodes[s].value=stringExists(variables,part_formula);
//      cout << " VARIABLE " << part_formula << endl;
      }
    else if (stringExists(functions,part_formula)!=-1)
      {
      formula_nodes[s].type=ParserNode::FUNCTION;
      formula_nodes[s].expression=part_formula;
      formula_nodes[s].value=stringExists(functions,part_formula);
//      cout << " FUNCTION " << part_formula << endl;
      }
    else
      {
// If the string is not understood by any of the above,
// Then it is considered as a new variable.
      formula_nodes[s].type=ParserNode::VARIABLE;
      formula_nodes[s].expression=part_formula;
      addVariable(part_formula);
      formula_nodes[s].value=stringExists(variables,part_formula);
/*      cout << "Value of "<<part_formula<< " : ";
      double temp;
      cin >> temp;
      setVariable(part_formula,temp);*/
      }
    }

//  for (int i=0;i<formula_nodes.size();i++)
//  cout << " " << formula_nodes[i].expression;
//  cout << endl;
  }

  
  
//////////////////////////////////////////////////////
//////////////////////////////////////////////////////
int Parser::stringExists(const vector< string>& vec, string & str)
  {
  int position=-1;
  for (int i=0; i<vec.size(); i++)
    {
    if (vec[i].compare(str)==0)
      {
      position=i;
      i=vec.size();
      }
    }
  return position;
 }


 
//////////////////////////////////////////////////////
//////////////////////////////////////////////////////
void Parser::upperCase(string& str)
  {
  for (unsigned int j = 0; j <str.size() ; j++)
    {
    char sj = str[j];
    if (sj >= 'a' && sj <= 'z') str[j] = (char)(sj - ('a' - 'A'));
    }
  }


  
//////////////////////////////////////////////////////
//////////////////////////////////////////////////////
void Parser::evaluate(double& result) 
  {
// 	for (int i=0; i<variables.size();i++)
// 	{
// 		cout << "variable " << variables[i] << " = " << variables_value[i] << endl;
// 	}
  evaluateNode(nodeRoot);
  result=nodeRoot->result;
  }

  
  
//////////////////////////////////////////////////////
//////////////////////////////////////////////////////
void Parser::evaluateNode(ParserNode* node) 
  {
  switch (node->type)
    {
    case 1: // OPERATION
      {
      evaluateNode(node->left);
      evaluateNode(node->right);
      evaluateNodeOperation(node);
      break;
      }
    case 2: // PARENTHESIS
    case 3: // VALUE
      {
      break;
      }
    case 4: // FUNCTION
      {
      evaluateNode(node->right);
      evaluateNodeFunction(node);
      break;
      }
    case 5: // VARIABLE
      {
      node->result=variables_value[int(node->value)];
      break;
      }
    default:
      {
      ERROR_IN_FILE;
      cerr << "Impossible case in Mathematical Parser with Node!" << endl;
      exit(-1);
      }
    }
  } 

  
  
//////////////////////////////////////////////////////
//////////////////////////////////////////////////////
void Parser::evaluateNodeOperation(ParserNode* node) 	
  {
  switch (int(node->value))
    {
    case 0: // -
      {
      node->result = ( node->left->result - node->right->result ); 
      break;
      }
    case 1: // +
      {
      node->result = ( node->left->result + node->right->result ); 
      break;
      }
    case 2: // /
      {
      node->result = ( node->left->result / node->right->result ); 
      break;
      }
    case 3: // *
      {
      node->result = ( node->left->result * node->right->result ); 
      break;
      }
    case 4: // <
      {
      if ( node->left->result < node->right->result )
        {
        node->result = 1.0; 
        }
      else
        {
        node->result = 0.0;
        }
      break;
      }
    case 5: // >
      {
      if ( node->left->result > node->right->result )
        {
        node->result = 1.0;
        }
      else
        {
        node->result = 0.0;
        }
      break;
      }
    case 6: // ^
      {
      node->result = ( pow(node->left->result,node->right->result) ); 
      break;
      }
    default:
      {
      ERROR_IN_FILE;
      cerr << "Impossible case in Mathematical Parser with Operator!" << endl;
      exit(-1);
      }
    }
  }


  
//////////////////////////////////////////////////////
//////////////////////////////////////////////////////
void Parser::evaluateNodeFunction(ParserNode* node) 	
  {
  switch (int(node->value))
    {
    case 0: // MINUS
      {
      node->result =  - node->right->result ; 
      break;
      }
    case 1: // COS
      {
      node->result = cos( node->right->result ); 
      break;
      }
    case 2: // SIN
      {
      node->result = sin( node->right->result ); 
      break;
      }
    case 3: // TAN
      {
      node->result = tan( node->right->result ); 
      break;
      }
    case 4: // ACOS
      {
      node->result = acos( node->right->result ); 
      break;
      }
    case 5: // ASIN
      {
      node->result = asin( node->right->result ); 
      break;
      }
    case 6: // ATAN
      {
      node->result = atan( node->right->result ); 
      break;
      }
    case 7: // COSH
      {
      node->result = cosh( node->right->result ); 
      break;
      }
    case 8: // SINH
      {
      node->result = sinh( node->right->result ); 
      break;
      }
    case 9: // TANH
      {
      node->result = tanh( node->right->result ); 
      break;
      }
    case 10: // ACOSH
      {
      node->result = acosh( node->right->result ); 
      break;
      }
    case 11: // ASINH
      {
      node->result = asinh( node->right->result ); 
      break;
      }
    case 12: // ATANH
      {
      node->result = atanh( node->right->result ); 
      break;
      }
    case 13: // EXP
      {
      node->result = exp( node->right->result ); 
      break;
      }
    case 14: // LOG
      {
      node->result = log( node->right->result ); 
      break;
      }
    case 15: // LOG10
      {
      node->result = log10( node->right->result ); 
      break;
      }
    case 16: // SQRT
      {
      node->result = sqrt( node->right->result ); 
      break;
      }
    case 17: // RAND
      {
      node->result = double(rand())/RAND_MAX;
      break;
      }
    default:
      {
      ERROR_IN_FILE;
      cerr << "Impossible case in Mathematical Parser with Function!" << endl;
      exit(-1);
      }
    }
  }
