// Copyright (C) 2004 by OpenFlower Team
// http://sourceforge.net/projects/openflower/
// Licensed under the GNU GPL Version 2.

// How to add a function (such as erf, atanh...) for the parser???
// It's very easy:
// 1. Add the function in the constructor Parser()
// 2. Add the definition in EvaluateNodeOperation(ParserNode* node);
// That's it :)
//
// An example of how to use the Mathematical Parser:
// Parser P1;
// P1.AddVariable("x");
// P1.AddVariable("y");
// fct_str="-2*x+1+exp(y)";
// P1.MakeTree(fct_str);
// P1.setVariable("x",0.3);
// P1.setVariable("y",-6.4);
// P1.Evaluate(result);
// cout << "P1(0.3, -6.4)= " << result;




#ifndef PARSER_H
#define PARSER_H

#include <stdlib.h>
#include <vector>
#include <string>
#include <deque>

#include "ParserNode.h"

using namespace std;

class Parser
{
public :
  Parser();
  ~Parser();

  void addVariable(string var);
  void setVariable(string var,const double var_value);
  void makeTree(string formula);
  void evaluate(double& result);
  void clear(); //!< Re-initialize all variables.

  inline const string & getExpression() {return expression;}

private :
  ParserNode* nodeRoot;   //!< root node of the tree.
  deque<ParserNode*> dq_op; //!< queue of operation.
  deque<ParserNode*> dq_ob; //!< queue of object (operand).
  vector<ParserNode> formula_nodes; //!< formula written as a vector of parser_node.
  vector<string> functions; //!< list of existing functions->defined in constructor.
  vector<string> variables; //!< list of existing variables->defined with addVariable.
  vector<double> variables_value; //!< same as above but with value instead of name.


  void cutFormula(const string& formula);  //!< cuts the formula and creates formula_nodes.
  int stringExists(const vector< string>& vec, string & str);  //!< searches if str is present in vec -> returns the position or -1 if not present.
  void upperCase(string& S);  //!< changes a string to uppercase.

  void makeTreeState0(const int pos, int& state);
  void makeTreeState1(const int pos, int& state);
  void makeTreeState2(const int pos, int& state);
  void de_queued();

  void evaluateNode(ParserNode* node);
  void evaluateNodeFunction(ParserNode* node);
  void evaluateNodeOperation(ParserNode* node);

  string expression;
  static const string OPERATIONS;  //!< list of existing operations.
  static const string PARENTHESIS; //!< list of existing parenthesis.

  };
#endif
