// Copyright (C) 2004 by OpenFlower Team
// http://sourceforge.net/projects/openflower/
// Licensed under the GNU GPL Version 2.

#include <iostream>

#include "Variable.h"
#include "MyMacros.h"

using namespace std;



////////////////////////////////////////////////////////////
// Creates a new empty Variable
////////////////////////////////////////////////////////////
Variable::Variable()
  {
  name = "empty";
  location="cell";    // default location value at center of control volumes
  nbValues = 1;
  nbDimensions0 = 1;
  nbDimensions1 = 1;
  }



Variable::Variable(const Variable& another)
  {
  name = another.name;
  location = another.location;
  nbValues = another.nbValues;
  nbDimensions0 = another.nbDimensions0;
  nbDimensions1 = another.nbDimensions1;
  values.resize(nbValues*nbDimensions0*nbDimensions1);
  for(int i=0;i<nbValues*nbDimensions0*nbDimensions1;i++) values[i] = another.values[i];
  }



////////////////////////////////////////////////////////////
// Creates a new Variable
////////////////////////////////////////////////////////////
Variable::Variable(const string the_name, const int nb_vals, const int dims0, const int dims1)
  {
  name = the_name; // name of the variable
  location="cell"; // Variable is located at center of control volumes or faces
  nbValues = nb_vals;
  nbDimensions0 = dims0;
  nbDimensions1 = dims1;
  values.resize(nbValues*nbDimensions0*nbDimensions1);
  for(int i=0;i<nbValues*nbDimensions0*nbDimensions1;i++) values[i] = 0.;
  }



void Variable::resize(const int _nbValues, const int dims0, const int dims1)
  {
  nbValues = _nbValues;
  nbDimensions0 = dims0;
  nbDimensions1 = dims1;

  assert(nbValues>0);
  assert(nbDimensions0>0);
  assert(nbDimensions1>0);
  
  values.resize(nbValues*nbDimensions0*nbDimensions1);
  for(int i=0;i<nbValues*nbDimensions0*nbDimensions1;i++) values[i] = 0.;
  }



void Variable::operator+=(const Variable& a)
  {
  if((a.location==location)
   &&(a.nbValues==nbValues)
   &&(a.nbDimensions0==nbDimensions0)
   &&(a.nbDimensions1==nbDimensions1))
    {
/*
    string s;
    s="_p_";
    name += s + a.name;
*/
    for(int i=0;i<nbValues*nbDimensions0*nbDimensions1;i++)
      {
      values[i] += a.values[i];
      }
    }
  else
    {
    ERROR_IN_FILE;
    cerr << "Trying to add two Variables of different types:"<<endl;
    cerr << "Variable "<<a.name << " with Variable " << name<<endl;
    assert(a.location==location);
    assert(a.nbValues==nbValues);
    exit(-1);
    }
  }



double Variable::getMin()
  {
  double min=1e30;
  for(int i=0;i<nbValues*nbDimensions0*nbDimensions1;i++)
    {
    if(values[i]<min) min=values[i];
    }
  return min;
  }



double Variable::getMax()
  {
  double max=-1e30;
  for(int i=0;i<nbValues*nbDimensions0*nbDimensions1;i++)
    {
    if(values[i]>max) max=values[i];
    }
  return max;
  }



void Variable::operator-=(const Variable& a)
  {
  if((a.location==location)
   &&(a.nbValues==nbValues)
   &&(a.nbDimensions0==nbDimensions0)
   &&(a.nbDimensions1==nbDimensions1))
    {
/*
    string s;
    s="_m_";
    name += s + a.name;
*/
    for(int i=0;i<nbValues*nbDimensions0*nbDimensions1;i++)
      {
      values[i] -= a.values[i];
      }
    }
  else
    {
    ERROR_IN_FILE;
    cerr << "Trying to add two Variables of different types:"<<endl;
    cerr << "Variable "<<a.name << " with Variable " << name<<endl;
    assert(a.location==location);
    assert(a.nbValues==nbValues);
    exit(-1);
    }
  }



void Variable::operator*=(const Variable& a)
  {
  if((a.location==location)
   &&(a.nbValues==nbValues)
   &&(a.nbDimensions0==nbDimensions0)
   &&(a.nbDimensions1==nbDimensions1))
    {
/*
    string s;
    s="_t_";
    name += s + a.name;
*/
    for(int i=0;i<nbValues*nbDimensions0*nbDimensions1;i++)
      {
      values[i] *= a.values[i];
      }
    }
  else
    {
    ERROR_IN_FILE;
    cerr << "Trying to add two Variables of different types:"<<endl;
    cerr << "Variable "<<a.name << " with Variable " << name<<endl;
    assert(a.location==location);
    assert(a.nbValues==nbValues);
    exit(-1);
    }
  }



void Variable::operator/=(const Variable& another)
  {
  if((another.location==location)
   &&(another.nbValues==nbValues)
   &&(another.nbDimensions0==nbDimensions0)
   &&(another.nbDimensions1==nbDimensions1))
    {
/*
    string s;
    s="_d_";
    name += s + another.name;
*/
    for(int i=0;i<nbValues*nbDimensions0*nbDimensions1;i++)
      {
      values[i] /= another.values[i];
      }
    }
  else
    {
    ERROR_IN_FILE;
    cerr << "Trying to add two Variables of different types:"<<endl;
    cerr << "Variable "<<another.name << " with Variable " << name<<endl;
    assert(another.location==location);
    assert(another.nbValues==nbValues);
    exit(-1);
    }
  }



void Variable::operator=(const Variable& another)
  {
  name = another.name;
  location = another.location;
  nbValues = another.nbValues;
  nbDimensions0 = another.nbDimensions0;
  nbDimensions1 = another.nbDimensions1;
  values.resize(nbValues*nbDimensions0*nbDimensions1);
    
  for(int i=0;i<nbValues*nbDimensions0*nbDimensions1;i++)
    {
    values[i] = another.values[i];
    }
  }

