// Copyright (C) 2004 by OpenFlower Team
// http://sourceforge.net/projects/openflower/
// Licensed under the GNU GPL Version 2.

#ifndef VARIABLE_H
#define VARIABLE_H

#include <stdlib.h>
#include <string>

#include <MyVector.h>

using namespace std;

class Variable
{
public:
  Variable();
  Variable(const Variable& another);
  Variable(const string _name, const int _nbValues, const int _nbDimensions0 = 1, const int _nbDimensions1 = 1);
  ~Variable(){};

  inline void setName(const string _name)  { name = _name; };
  inline void setLocation(const string _location) { location = _location; };
  inline void setNbValues(const int _nbValues) { nbValues = _nbValues; };
  inline void setNbDimensions0(const int _nbDimensions0) { nbDimensions0 = _nbDimensions0; };
  inline void setNbDimensions1(const int _nbDimensions1) { nbDimensions1 = _nbDimensions1; };
  
  inline string getName() const { return name; };
  inline string getLocation() const { return location; };
  inline int getNbValues() const { return nbValues; };
  inline int getNbDimensions0() const { return nbDimensions0; };
  inline int getNbDimensions1() const { return nbDimensions1; };
  
  void resize(const int nb, const int i = 1, const int j = 1);

  inline void setValue(double Val, int nb, int i = 0, int j = 0) {values[nb*nbDimensions0*nbDimensions1+i*nbDimensions1+j]=Val;};
  inline double getValue(int nb, int i = 0, int j = 0)const {return values[nb*nbDimensions0*nbDimensions1+i*nbDimensions1+j];};
  inline void addValue(double Val, int nb, int i = 0, int j = 0) {values[nb*nbDimensions0*nbDimensions1+i*nbDimensions1+j]+=Val;};

  double getMin();
  double getMax();


  void operator += (const Variable& another);
  void operator -= (const Variable& another);
  void operator *= (const Variable& another);
  void operator /= (const Variable& another);
  void operator  = (const Variable& another);


  inline void operator += (const double a)
    {
    for(unsigned int i=0;i<nbValues*nbDimensions0*nbDimensions1;i++) values[i] += a;
    };
  inline void operator -= (const double a)
    {
    for(unsigned int i=0;i<nbValues*nbDimensions0*nbDimensions1;i++) values[i] -= a;
    };
  inline void operator *= (const double a)
    {
    for(unsigned int i=0;i<nbValues*nbDimensions0*nbDimensions1;i++) values[i] *= a;
    };
  inline void operator /= (const double a)
    {
    assert(a!=0.);
    double b=1./a;
    for(unsigned int i=0;i<nbValues*nbDimensions0*nbDimensions1;i++) values[i] *= b;
    };

  
protected:
  string name;
  string location;
  unsigned int nbValues;
  unsigned int nbDimensions0;
  unsigned int nbDimensions1;

  MyVector<double> values;

};

#endif
